@@ -1,7 +1,7 @@
 /**
  * @file browser.c  Launching different external browsers
  *
- * Copyright (C) 2003-2010 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2015 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,32 +34,38 @@ static struct browser browsers[] = {
 		NULL, NULL,
 		NULL, NULL
 	},
-	{
-		/* tested with SeaMonkey 1.0.6 */
-		"mozilla", "Mozilla", "mozilla %s",
-		NULL, "mozilla -remote openURL(%s)",
-		NULL, "mozilla -remote 'openURL(%s,new-window)'",
-		NULL, "mozilla -remote 'openURL(%s,new-tab)'"
-	},
 	{
 		/* tested with Firefox 1.5 and 2.0 */
-		"firefox", "Firefox","firefox \"%s\"",
-		NULL, "firefox -a firefox -remote \"openURL(%s)\"",
-		NULL, "firefox -a firefox -remote 'openURL(%s,new-window)'",
-		NULL, "firefox -a firefox -remote 'openURL(%s,new-tab)'"
+		"firefox", "Firefox", "firefox \"%s\"",
+		"firefox \"%s\"", NULL,
+		"firefox -a firefox -new-window \"%s\"", NULL,
+		"firefox -a firefox -new-tab \"%s\"", NULL
 	},
 	{
-		"opera", "Opera","opera \"%s\"",
+		"google-chrome", "Chrome", "google-chrome \"%s\"",
+		NULL, NULL,
+		NULL, NULL,
+		NULL, NULL
+	},		
+	{
+		"opera", "Opera", "opera \"%s\"",
 		"opera \"%s\"", "opera -remote \"openURL(%s)\"",
 		"opera -newwindow \"%s\"", NULL,
 		"opera -newpage \"%s\"", NULL
 	},
 	{
-		"epiphany", "Epiphany","epiphany \"%s\"",
+		"epiphany", "Epiphany", "epiphany \"%s\"",
 		NULL, NULL,
 		"epiphany \"%s\"", NULL,
 		"epiphany -n \"%s\"", NULL
 	},
+	{
+		/* tested with SeaMonkey 1.0.6 */
+		"mozilla", "Mozilla", "mozilla %s",
+		NULL, "mozilla -remote openURL(%s)",
+		NULL, "mozilla -remote 'openURL(%s,new-window)'",
+		NULL, "mozilla -remote 'openURL(%s,new-tab)'"
+	},
 	{
 		"konqueror", "Konqueror", "kfmclient openURL \"%s\"",
 		NULL, NULL,
@@ -135,7 +141,7 @@ browser_get_command (struct browser *browser, gboolean remote, gboolean fallback
 	}
 	g_free (libname);
 		
-	return cmd?g_strdup (cmd):NULL;
+	return g_strdup (cmd);
 }
 
 /** 
@@ -238,6 +244,7 @@ browser_launch_URL_external (const gchar *uri)
 	struct browser	*browser;
 	gchar		*cmd = NULL;
 	gboolean	done = FALSE;	
+	gchar		*libname = NULL;
 	
 	g_assert (uri != NULL);
 	
@@ -249,8 +256,12 @@ browser_launch_URL_external (const gchar *uri)
 			done = browser_execute (cmd, uri, TRUE);
 			g_free (cmd);
 		} else {
-			/* the "default" browser has no command to use the GTK launch mechanism */
-			done = gtk_show_uri (NULL, uri, 0, NULL);
+			/* the "default" browser has no command to use the GTK
+			   launch mechanism, so we use gtk_show_uri() instead */
+			conf_get_str_value (BROWSER_ID, &libname);
+			if (g_str_equal (libname, "default"))
+				done = gtk_show_uri (NULL, uri, 0, NULL);
+			g_free (libname);
 		}
 	}
 	
