@@ -1,7 +1,7 @@
 /**
  * @file db.c sqlite backend
  * 
- * Copyright (C) 2007-2012  Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2012  Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
    http://lzone.de/wiki/doku.php?id=liferea:v1.8:db_schema */
 
 static sqlite3	*db = NULL;
+gboolean searchFolderRebuild = FALSE;
 
 /** hash of all prepared statements */
 static GHashTable *statements = NULL;
@@ -105,13 +106,14 @@ db_table_exists (const gchar *name)
 {
 	gchar		*sql;
 	sqlite3_stmt	*stmt;
-	gint		res;
+	gint		res = 0;
 
 	sql = sqlite3_mprintf ("SELECT COUNT(type) FROM sqlite_master WHERE type = 'table' AND name = '%s';", name);
 	db_prepare_stmt (&stmt, sql);
 	sqlite3_reset (stmt);
-	sqlite3_step (stmt);
-	res = sqlite3_column_int (stmt, 0);
+	if (SQLITE_ROW == sqlite3_step (stmt))
+		res = sqlite3_column_int (stmt, 0);
+	
 	sqlite3_finalize (stmt);
 	sqlite3_free (sql);
 	return (1 == res);
@@ -132,7 +134,7 @@ db_set_schema_version (gint schemaVersion)
 static gint
 db_get_schema_version (void)
 {
-	guint		schemaVersion;
+	guint		schemaVersion = 0;
 	sqlite3_stmt	*stmt;
 	
 	if (!db_table_exists ("info")) {
@@ -145,8 +147,8 @@ db_get_schema_version (void)
 	}
 	
 	db_prepare_stmt (&stmt, "SELECT value FROM info WHERE name = 'schemaVersion'");
-	sqlite3_step (stmt);
-	schemaVersion = sqlite3_column_int (stmt, 0);
+	if (SQLITE_ROW == sqlite3_step (stmt))
+		schemaVersion = sqlite3_column_int (stmt, 0);
 	sqlite3_finalize (stmt);
 	
 	return schemaVersion;
@@ -230,10 +232,10 @@ db_vacuum (void)
 static void
 db_open (void)
 {
-	gchar		*filename;
-	gint		res;
+	gchar	*filename;
+	gint	res;
 
-	filename = common_create_cache_filename (NULL, "liferea", "db");
+	filename = common_create_data_filename ("liferea.db");
 	debug1 (DEBUG_DB, "Opening DB file %s...", filename);
 	res = sqlite3_open_v2 (filename, &db, SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE, NULL);
 	if (SQLITE_OK != res)
@@ -247,14 +249,13 @@ db_open (void)
 	db_exec("PRAGMA synchronous=NORMAL");
 }
 
-#define SCHEMA_TARGET_VERSION 9
+#define SCHEMA_TARGET_VERSION 10
 
 /* opening or creation of database */
 void
 db_init (void)
 {
 	gint		res;
-	GError          *error;
 		
 	debug_enter ("db_init");
 
@@ -417,6 +418,22 @@ db_init (void)
 				sqlite3_finalize (stmt);
 			}
 		}
+
+		if (db_get_schema_version () == 9) {
+			/* A parent node id to search folder relation to allow cleanups */
+			db_exec ("BEGIN; "
+			         "DROP TABLE search_folder_items; "
+				 "CREATE TABLE search_folder_items ("
+				 "   node_id            STRING,"
+				 "   parent_node_id     STRING,"
+	         		 "   item_id		INTEGER,"
+				 "   PRIMARY KEY (node_id, item_id)"
+				 ");"
+			         "REPLACE INTO info (name, value) VALUES ('schemaVersion',10); "
+			         "END;" );
+
+			searchFolderRebuild = TRUE;
+		}
 	}
 
 	if (SCHEMA_TARGET_VERSION != db_get_schema_version ())
@@ -505,6 +522,7 @@ db_init (void)
 
 	db_exec ("CREATE TABLE search_folder_items ("
 	         "   node_id            STRING,"
+	         "   parent_node_id     STRING,"
 	         "   item_id		INTEGER,"
 		 "   PRIMARY KEY (node_id, item_id)"
 		 ");");
@@ -536,9 +554,20 @@ db_init (void)
 		 "END;");
         
 	debug0 (DEBUG_DB, "Checking for search folder items without a feed list node...\n");
+	db_exec ("DELETE FROM search_folder_items WHERE parent_node_id NOT IN "
+        	 "(SELECT node_id FROM node);");
+
+	debug0 (DEBUG_DB, "Checking for search folder items without a search folder...\n");
 	db_exec ("DELETE FROM search_folder_items WHERE node_id NOT IN "
         	 "(SELECT node_id FROM node);");
-			  
+
+	debug0 (DEBUG_DB, "Checking for search folder with comments...\n");
+	db_exec ("DELETE FROM search_folder_items WHERE comment = 1;");
+
+	debug0 (DEBUG_DB, "Checking for subscription metadata without node...\n");
+	db_exec ("DELETE FROM subscription_metadata WHERE node_id NOT IN "
+          	 "(SELECT node_id FROM node);");
+
 	debug0 (DEBUG_DB, "DB cleanup finished. Continuing startup.");
 		
 	/* 4. Creating triggers (after cleanup so it is not slowed down by triggers) */
@@ -554,6 +583,7 @@ db_init (void)
         	 "BEGIN "
 		 "   DELETE FROM node WHERE node_id = old.node_id; "
 		 "   DELETE FROM subscription_metadata WHERE node_id = old.node_id; "
+		 "   DELETE FROM search_folder_items WHERE parent_node_id = old.node_id; "
         	 "END;");
 
 	/* Note: view counting triggers are set up in the view preparation code (see db_view_create()) */		
@@ -563,7 +593,7 @@ db_init (void)
 	                  "SELECT item_id FROM items WHERE node_id = ?");
 
 	db_new_statement ("itemsetLoadOffsetStmt",
-			  "SELECT item_id FROM items WHERE item_id >= ? limit ?");
+			  "SELECT item_id FROM items WHERE comment = 0 LIMIT ? OFFSET ?");
 		       
 	db_new_statement ("itemsetReadCountStmt",
 	                  "SELECT COUNT(item_id) FROM items "
@@ -679,32 +709,38 @@ db_init (void)
 	                  "REPLACE INTO node (node_id,parent_id,title,type,expanded,view_mode,sort_column,sort_reversed) VALUES (?,?,?,?,?,?,?,?)");
 	                  
 	db_new_statement ("itemUpdateSearchFoldersStmt",
-	                  "REPLACE INTO search_folder_items (node_id, item_id) VALUES (?,?)");
+	                  "REPLACE INTO search_folder_items (node_id, parent_node_id, item_id) VALUES (?,?,?)");
+
+	db_new_statement ("itemRemoveFromSearchFolderStmt",
+	                  "DELETE FROM search_folder_items WHERE node_id =? AND item_id = ?;");
 	                  
 	db_new_statement ("searchFolderLoadStmt",
 	                  "SELECT item_id FROM search_folder_items WHERE node_id = ?;");
+
+	db_new_statement ("searchFolderCountStmt",
+	                  "SELECT count(item_id) FROM search_folder_items WHERE node_id = ?;");
+
+	db_new_statement ("nodeIdListStmt",
+	                  "SELECT node_id FROM node;");
+
+	db_new_statement ("nodeRemoveStmt",
+	                  "DELETE FROM node WHERE node_id = ?;");
 			  
 	g_assert (sqlite3_get_autocommit (db));
 	
 	debug_exit ("db_init");
 }
 
-static void
-db_free_statements (gpointer key, gpointer value, gpointer user_data)
-{
-	value = NULL;
-}
-
 void
 db_deinit (void) 
 {
+
 	debug_enter ("db_deinit");
 	
 	if (FALSE == sqlite3_get_autocommit (db))
 		g_warning ("Fatal: DB not in auto-commit mode. This is a bug. Data may be lost!");
 	
 	if (statements) {
-		g_hash_table_foreach (statements, db_free_statements, NULL);
 		g_hash_table_destroy (statements);	
 		statements = NULL;
 	}
@@ -861,7 +897,7 @@ db_item_load (gulong id)
 
 	if (sqlite3_step (stmt) == SQLITE_ROW) {
 		item = db_load_item_from_columns (stmt);
-		sqlite3_step (stmt);
+		(void) sqlite3_step (stmt);
 	} else {
 		debug1 (DEBUG_DB, "Could not load item with id %lu!", id);
 	}
@@ -920,28 +956,52 @@ db_item_search_folders_update (itemPtr item)
 	sqlite3_stmt	*stmt;
 	gint 		res;
 	GSList		*iter, *list;
+
+	/* Bail on comments which are not covered by search folders */
+	if (item->isComment)
+		return;
 	
-	// FIXME: also remove from search folders
+	/* Add item to all search folders it now belongs to */
 
 	stmt = db_get_statement ("itemUpdateSearchFoldersStmt");
-	iter = list = vfolder_get_all_with_item_id (item->id);
+	iter = list = vfolder_get_all_with_item_id (item);
 	while (iter) {
 		vfolderPtr vfolder = (vfolderPtr)iter->data;
+		sqlite3_reset (stmt);
+		sqlite3_bind_text (stmt, 1, vfolder->node->id, -1, SQLITE_TRANSIENT);
+		sqlite3_bind_text (stmt, 2, item->nodeId, -1, SQLITE_TRANSIENT);
+		sqlite3_bind_int (stmt, 3, item->id);
+		res = sqlite3_step (stmt);
 
+		if (SQLITE_DONE != res) 
+			g_warning ("item add to search folder failed (error code=%d, %s)", res, sqlite3_errmsg (db));
+		iter = g_slist_next (iter);
+
+	}
+	g_slist_free (list);
+
+	sqlite3_finalize (stmt);
+
+	/* Remove item from all search folders it does not belong
+	   (we do not check if it is in there, just remove it) */
+
+	stmt = db_get_statement ("itemRemoveFromSearchFolderStmt");
+	iter = list = vfolder_get_all_without_item_id (item);
+	while (iter) {
+		vfolderPtr vfolder = (vfolderPtr)iter->data;
 		sqlite3_reset (stmt);
 		sqlite3_bind_text (stmt, 1, vfolder->node->id, -1, SQLITE_TRANSIENT);
 		sqlite3_bind_int (stmt, 2, item->id);
 		res = sqlite3_step (stmt);
 
 		if (SQLITE_DONE != res) 
-			g_warning ("item update of search folders failed (error code=%d, %s)", res, sqlite3_errmsg (db));
+			g_warning ("item remove from search folder failed (error code=%d, %s)", res, sqlite3_errmsg (db));
 		iter = g_slist_next (iter);
 
 	}
-	g_slist_free (iter);
+	g_slist_free (list);
 
 	sqlite3_finalize (stmt);
-
 }
 
 void
@@ -971,7 +1031,7 @@ db_item_update (itemPtr item)
 	sqlite3_bind_text (stmt, 6,  item->source, -1, SQLITE_TRANSIENT);
 	sqlite3_bind_text (stmt, 7,  item->sourceId, -1, SQLITE_TRANSIENT);
 	sqlite3_bind_int  (stmt, 8,  item->validGuid?1:0);
-	sqlite3_bind_text (stmt, 9, item->description, -1, SQLITE_TRANSIENT);
+	sqlite3_bind_text (stmt, 9,  item->description, -1, SQLITE_TRANSIENT);
 	sqlite3_bind_int  (stmt, 10, item->time);
 	sqlite3_bind_text (stmt, 11, item->commentFeedId, -1, SQLITE_TRANSIENT);
 	sqlite3_bind_int  (stmt, 12, item->isComment?1:0);
@@ -1137,16 +1197,16 @@ db_itemset_mark_all_popup (const gchar *id)
 }
 
 gboolean
-db_itemset_get (itemSetPtr itemSet, gulong id, guint limit)
+db_itemset_get (itemSetPtr itemSet, gulong offset, guint limit)
 {
 	sqlite3_stmt	*stmt;
 	gboolean	success = FALSE;
 
-	debug2 (DEBUG_DB, "loading %d items starting with %lu", limit, id);
+	debug2 (DEBUG_DB, "loading %d items offset %lu", limit, offset);
 
 	stmt = db_get_statement ("itemsetLoadOffsetStmt");
-	sqlite3_bind_int (stmt, 1, id);
-	sqlite3_bind_int (stmt, 2, limit);
+	sqlite3_bind_int (stmt, 1, limit);
+	sqlite3_bind_int (stmt, 2, offset);
 
 	while (sqlite3_step (stmt) == SQLITE_ROW) {
 		itemSet->ids = g_list_append (itemSet->ids, GUINT_TO_POINTER (sqlite3_column_int (stmt, 0)));
@@ -1294,7 +1354,7 @@ db_search_folder_load (const gchar *id)
 	stmt = db_get_statement ("searchFolderLoadStmt");
 	res = sqlite3_bind_text (stmt, 1, id, -1, SQLITE_TRANSIENT);
 	if (SQLITE_OK != res)
-		g_error ("db_load_metadata: sqlite bind failed (error code %d)!", res);
+		g_error ("db_search_folder_load: sqlite bind failed (error code %d)!", res);
 	
 	itemSet = g_new0 (struct itemSet, 1);
 	itemSet->nodeId = (gchar *)id;
@@ -1346,7 +1406,8 @@ db_search_folder_add_items (const gchar *id, GSList *items)
 
 		sqlite3_reset (stmt);
 		sqlite3_bind_text (stmt, 1, id, -1, SQLITE_TRANSIENT);
-		sqlite3_bind_int (stmt, 2, item->id);
+		sqlite3_bind_text (stmt, 2, item->nodeId, -1, SQLITE_TRANSIENT);
+		sqlite3_bind_int (stmt, 3, item->id);
 		res = sqlite3_step (stmt);
 		if (SQLITE_DONE != res)
 			g_error ("db_search_folder_add_items: sqlite3_step (error code %d)!", res);
@@ -1360,8 +1421,33 @@ db_search_folder_add_items (const gchar *id, GSList *items)
 	debug0 (DEBUG_DB, "adding items to search folder finished");
 }
 
+guint 
+db_search_folder_get_item_count (const gchar *id) 
+{
+	sqlite3_stmt	*stmt;
+	gint		res;
+	guint		count = 0;
+	
+	debug_start_measurement (DEBUG_DB);
+	
+	stmt = db_get_statement ("searchFolderCountStmt");
+	sqlite3_bind_text (stmt, 1, id, -1, SQLITE_TRANSIENT);
+	res = sqlite3_step (stmt);
+	
+	if (SQLITE_ROW == res)
+		count = sqlite3_column_int (stmt, 0);
+	else
+		g_warning("item read counting failed (error code=%d, %s)", res, sqlite3_errmsg (db));
+		
+	sqlite3_finalize (stmt);
+
+	debug_end_measurement (DEBUG_DB, "counting unread items");
+
+	return count;
+}
+
 static GSList *
-db_subscription_metadata_load(const gchar *id) 
+db_subscription_metadata_load (const gchar *id) 
 {
 	GSList		*metadata = NULL;
 	sqlite3_stmt	*stmt;
@@ -1370,7 +1456,7 @@ db_subscription_metadata_load(const gchar *id)
 	stmt = db_get_statement ("subscriptionMetadataLoadStmt");
 	res = sqlite3_bind_text (stmt, 1, id, -1, SQLITE_TRANSIENT);
 	if (SQLITE_OK != res)
-		g_error ("db_load_metadata: sqlite bind failed (error code %d)!", res);
+		g_error ("db_subscription_metadata_load: sqlite bind failed (error code %d)!", res);
 
 	while (sqlite3_step (stmt) == SQLITE_ROW) {
 		metadata = db_metadata_list_append (metadata, sqlite3_column_text(stmt, 0), 
@@ -1399,10 +1485,9 @@ db_subscription_metadata_update_cb (const gchar *key,
 	sqlite3_bind_text (stmt, 4, value, -1, SQLITE_TRANSIENT);
 	res = sqlite3_step (stmt);
 	if (SQLITE_DONE != res) 
-		g_warning ("Update in \"metadata\" table failed (error code=%d, %s)", res, sqlite3_errmsg (db));
+		g_warning ("Update in \"subscription_metadata\" table failed (error code=%d, %s)", res, sqlite3_errmsg (db));
 
 	sqlite3_finalize (stmt);
-
 }
 
 static void
@@ -1440,13 +1525,13 @@ db_subscription_update (subscriptionPtr subscription)
 	
 	res = sqlite3_step (stmt);
 	if (SQLITE_DONE != res)
-		g_warning ("Could not update subscription info %s in DB (error code %d)!", subscription->node->id, res);
-
+		g_warning ("Could not update subscription info for node id %s in DB (error code %d)!", subscription->node->id, res);
+	
 	sqlite3_finalize (stmt);
 
 	db_subscription_metadata_update (subscription);
 		
-	debug_end_measurement (DEBUG_DB, "subscription_update");
+	debug_end_measurement (DEBUG_DB, "subscription update");
 }
 
 void
@@ -1491,9 +1576,64 @@ db_node_update (nodePtr node)
 	
 	res = sqlite3_step (stmt);
 	if (SQLITE_DONE != res)
-		g_warning ("Could not update subscription info %s in DB (error code %d)!", node->id, res);
+		g_warning ("Could not update node info %s in DB (error code %d)!", node->id, res);
+
+	sqlite3_finalize (stmt);
+		
+	debug_end_measurement (DEBUG_DB, "node update");
+}
+
+static gboolean
+db_node_find (nodePtr node, gpointer id)
+{
+	GSList *iter;
+
+	if (g_str_equal (node->id, (gchar *)id))
+		return TRUE;
+
+	iter = node->children;
+	while (iter) {
+		if (db_node_find ((nodePtr)iter->data, id))
+			return TRUE;
+		iter = g_slist_next (iter);
+	}
+
+	return FALSE;
+}
+
+static void
+db_node_remove (const gchar *id)
+{
+	sqlite3_stmt	*stmt;
+	gint		res;
+
+	stmt = db_get_statement ("nodeRemoveStmt");	
+	sqlite3_bind_text (stmt, 1, id, -1, SQLITE_TRANSIENT);
+
+	res = sqlite3_step (stmt);
+	if (SQLITE_DONE != res)
+		g_warning ("Could not remove node %s in DB (error code %d)!", id, res);
+
+	sqlite3_finalize (stmt);
+}
+
+void
+db_node_cleanup (nodePtr root)
+{
+	sqlite3_stmt	*stmt;
+
+	debug0 (DEBUG_DB, "Cleaning node ids...");
+
+	/* Fetch all node ids */
+	stmt = db_get_statement ("nodeIdListStmt");
+	while (sqlite3_step (stmt) == SQLITE_ROW) {
+		/* Drop node ids not in feed list anymore */
+		const gchar *id = sqlite3_column_text (stmt, 0);
+		if (id && !db_node_find (root, (gpointer)id)) {
+			db_subscription_remove (id);	/* in case it is a subscription */
+			db_node_remove (id);		/* in case it is a folder */
+		}
+	}
 
 	sqlite3_finalize (stmt);
-
-	debug_end_measurement (DEBUG_DB, "subscription_update");
 }
