@@ -2,11 +2,12 @@
  * @file favicon.c Liferea favicon handling
  * 
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
+ * Copyright (C) 2015 Lars Windolf <lars.windolf@gmx.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,6 +36,7 @@
 #include "favicon.h"
 #include "feed.h"
 #include "html.h"
+#include "metadata.h"
 
 typedef struct faviconDownloadCtxt {
 	gchar		        *id;		/**< favicon cache id */
@@ -67,31 +69,28 @@ favicon_download_ctxt_free (faviconDownloadCtxtPtr ctxt)
 
 static void favicon_download_run(faviconDownloadCtxtPtr ctxt);
 
-GdkPixbuf * favicon_load_from_cache(const gchar *id) {
+GdkPixbuf *
+favicon_load_from_cache (const gchar *id)
+{
 	struct stat	statinfo;
 	gchar		*filename;
 	GdkPixbuf	*pixbuf, *result = NULL;
 	GError 		*error = NULL;
 
-	debug_enter("favicon_load_from_cache");
+	filename = common_create_cache_filename ("favicons", id, "png");
 	
-	/* try to load a saved favicon */
-	filename = common_create_cache_filename("cache" G_DIR_SEPARATOR_S "favicons", id, "png");
-	
-	if(0 == stat((const char*)filename, &statinfo)) {
-		pixbuf = gdk_pixbuf_new_from_file(filename, &error);
-		if(pixbuf) {
-			result = gdk_pixbuf_scale_simple(pixbuf, 16, 16, GDK_INTERP_BILINEAR);
-			g_object_unref(pixbuf);
+	if (0 == stat ((const char*)filename, &statinfo)) {
+		pixbuf = gdk_pixbuf_new_from_file (filename, &error);
+		if (pixbuf && !error) {
+			result = gdk_pixbuf_scale_simple (pixbuf, 16, 16, GDK_INTERP_BILINEAR);
+			g_object_unref (pixbuf);
 		} else { /* Error */
-			fprintf(stderr, "Failed to load pixbuf file: %s: %s\n",
+			fprintf (stderr, "Failed to load pixbuf file: %s: %s\n",
 			        filename, error->message);
-			g_error_free(error);
+			g_error_free (error);
 		}
 	}
-	g_free(filename);	
-
-	debug_exit("favicon_load_from_cache");
+	g_free (filename);
 	
 	return result;
 }
@@ -114,7 +113,7 @@ void favicon_remove_from_cache(const gchar *id) {
 	debug_enter("favicon_remove");
 	
 	/* try to load a saved favicon */
-	filename = common_create_cache_filename( "cache" G_DIR_SEPARATOR_S "favicons", id, "png");
+	filename = common_create_cache_filename ("favicons", id, "png");
 	if(g_file_test(filename, G_FILE_TEST_EXISTS)) {
 		if(0 != unlink(filename))
 			g_warning ("Removal of %s failed", filename);
@@ -145,7 +144,7 @@ favicon_download_icon_cb (const struct updateResult * const result, gpointer use
 			if (gdk_pixbuf_loader_close (loader, &err)) {
 				pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
 				if (pixbuf) {
-					tmp = common_create_cache_filename ("cache" G_DIR_SEPARATOR_S "favicons", ctxt->id, "png");
+					tmp = common_create_cache_filename ("favicons", ctxt->id, "png");
 					debug2 (DEBUG_UPDATE, "saving favicon %s to file %s", ctxt->id, tmp);
 					if (!gdk_pixbuf_save (pixbuf, tmp, "png", &err, NULL)) {
 						g_warning ("Could not save favicon (id=%s) to file %s!", ctxt->id, tmp);
@@ -168,7 +167,7 @@ favicon_download_icon_cb (const struct updateResult * const result, gpointer use
 		}
 
 		if (err) {
-			g_warning ("%s\n", err->message);
+			debug1 (DEBUG_UPDATE, "%s", err->message);
 			g_error_free (err);
 		}
 
@@ -252,56 +251,78 @@ static gint count_slashes(const gchar *str) {
 	return slashes;
 }
 
-void favicon_download(const gchar *id, const gchar *html_url, const gchar *source_url, const updateOptionsPtr options, faviconUpdatedCb callback, gpointer user_data) {
+void
+favicon_download (subscriptionPtr subscription,
+		  const gchar *html_url,
+		  const gchar *source_url,
+		  const updateOptionsPtr options,
+		  faviconUpdatedCb callback,
+		  gpointer user_data)
+{
+	const gchar		*id;
 	faviconDownloadCtxtPtr	ctxt;
 	gchar			*tmp, *tmp2;
-	
+
 	debug_enter("favicon_download");
-	
+
+	id = subscription->node->id;
+
 	ctxt = favicon_download_ctxt_new ();
 	ctxt->id = g_strdup (id);
 	ctxt->options = update_options_copy (options);
 	ctxt->callback = callback;
 	ctxt->user_data = user_data;
-	
+
 	/*
 	 * This code tries to download from a series of URLs. If there are no
 	 * favicons, this will make five downloads, three of which will be 404
 	 * errors. Hopefully this will not cause any webservers pain because
 	 * this code should be run only once a month per feed.
 	 *
-	 * 1. --> downloading HTML of the feed url and looking for a favicon reference
-	 * 2. --> downloading HTML of root of webserver and looking for a favicon reference
-	 * 3. --> downloading favicon from the root HTML
-	 * 4. --> downloading favicon from directory of RSS feed
-	 * 5. --> downloading favicon from root of webserver of the RSS feed
+	 * 1. --> downloading favicon from the feed (e.g. <icon> tag in atom feeds)
+	 * 2. --> downloading HTML of the feed url and looking for a favicon reference
+	 * 3. --> downloading HTML of root of webserver and looking for a favicon reference
+	 * 4. --> downloading favicon from the root HTML
+	 * 5. --> downloading favicon from directory of RSS feed
+	 * 6. --> downloading favicon from root of webserver of the RSS feed
 	 */
-	  
+
 	/* In the following These URLs will be prepared here and passed as a list to 
 	   the download function that will then process in order
 	   until success or the end of the list is reached... */
 	debug1(DEBUG_UPDATE, "preparing download URLs for favicon %s...", ctxt->id);
-	
+
 	/* case 1. */
-	if(html_url) {
-		tmp = g_strstrip (g_strdup(html_url));
-		ctxt->urls = g_slist_append(ctxt->urls, tmp);
-		debug1(DEBUG_UPDATE, "(1) adding favicon search URL: %s", tmp);
+	if (metadata_list_get (subscription->metadata, "icon")) {
+		tmp = g_strstrip (g_strdup (metadata_list_get (subscription->metadata, "icon")));
+		ctxt->urls = g_slist_append (ctxt->urls, tmp);
+		debug1 (DEBUG_UPDATE, "(1) adding favicon search URL: %s", tmp);
 	}
 
 	/* case 2. */
-	g_assert(source_url);	
+	if(html_url) {
+		tmp = g_strstrip (g_strdup (html_url));
+		ctxt->urls = g_slist_append(ctxt->urls, tmp);
+		debug1(DEBUG_UPDATE, "(2) adding favicon search URL: %s", tmp);
+	}
+
+	/* case 3. */
+	g_assert(source_url);
 	if(*source_url != '|') {
-		tmp = tmp2 = g_strstrip (g_strdup(source_url));
+		tmp = tmp2 = g_strstrip (g_strdup (source_url));
+
+		if(tmp[strlen(tmp) - 1] == '/')
+			tmp[strlen(tmp) - 1] = 0;	/* Strip trailing slash */
+
 		tmp = strrchr(tmp, '/');
 		if(tmp) {
 			*tmp = 0;
 			ctxt->urls = g_slist_append(ctxt->urls, tmp2);
-			debug1(DEBUG_UPDATE, "(2) adding favicon search URL: %s", tmp2);
+			debug1(DEBUG_UPDATE, "(3) adding favicon search URL: %s", tmp2);
 		}
 	}
-	
-	/* case 3. */
+
+	/* case 4. */
 	if(html_url) {
 		if(2 < count_slashes(html_url)) {
 			tmp = tmp2 = g_strstrip (g_strdup (html_url));
@@ -313,15 +334,15 @@ void favicon_download(const gchar *id, const gchar *html_url, const gchar *sourc
 					tmp = tmp2;
 					tmp2 = g_strdup_printf("%s/favicon.ico", tmp);
 					ctxt->urls = g_slist_append(ctxt->urls, tmp2);
-					debug1(DEBUG_UPDATE, "(3) adding favicon source URL: %s", tmp2);
+					debug1(DEBUG_UPDATE, "(4) adding favicon source URL: %s", tmp2);
 				}
 			}
 			g_free(tmp);
 		}
 	}
-	
+
 	if(*source_url != '|' && 2 < count_slashes(source_url)) {
-		/* case 4 */
+		/* case 5 */
 		tmp = tmp2 = g_strstrip (g_strdup (source_url));
 		tmp = strrchr(tmp, '/');
 		if(tmp) {
@@ -329,11 +350,11 @@ void favicon_download(const gchar *id, const gchar *html_url, const gchar *sourc
 			tmp = tmp2;
 			tmp2 = g_strdup_printf("%s/favicon.ico", tmp);
 			ctxt->urls = g_slist_append(ctxt->urls, tmp2);
-			debug1(DEBUG_UPDATE, "(4) adding favicon source URL: %s", tmp2);
+			debug1(DEBUG_UPDATE, "(5) adding favicon source URL: %s", tmp2);
 		}
 		g_free(tmp);
-	
-		/* case 5 */
+
+		/* case 6 */
 		tmp = tmp2 = g_strstrip (g_strdup (source_url));
 		tmp = strstr(tmp, "://");
 		if(tmp) {
@@ -343,13 +364,13 @@ void favicon_download(const gchar *id, const gchar *html_url, const gchar *sourc
 				tmp = tmp2;
 				tmp2 = g_strdup_printf("%s/favicon.ico", tmp);
 				ctxt->urls = g_slist_append(ctxt->urls, tmp2);
-				debug1(DEBUG_UPDATE, "(5) adding favicon source URL: %s", tmp2);
+				debug1(DEBUG_UPDATE, "(6) adding favicon source URL: %s", tmp2);
 			}
 		}
 		g_free(tmp);
 	}
-			
+
 	favicon_download_run(ctxt);
-	
+
 	debug_exit("favicon_download");
 }
