@@ -1,7 +1,7 @@
 /**
  * @file feed.c  feed node and subscription type
  * 
- * Copyright (C) 2003-2011 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2011 Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -35,11 +35,10 @@
 #include "render.h"
 #include "update.h"
 #include "xml.h"
-#include "ui/auth_dialog.h"
 #include "ui/icons.h"
 #include "ui/liferea_shell.h"
 #include "ui/subscription_dialog.h"
-#include "ui/ui_node.h"
+#include "ui/feed_list_node.h"
 #include "notification/notification.h"
 
 feedPtr
@@ -283,7 +282,7 @@ feed_process_update_result (subscriptionPtr subscription, const struct updateRes
 
 			liferea_shell_set_status_bar (_("\"%s\" updated..."), node_get_title (node));
 
-			if (!feed->preventPopup)
+			if (!feed->preventPopup && newCount)
 				notification_node_has_new_items (node, feed->enforcePopup);
 		}
 
@@ -329,7 +328,7 @@ feed_update_counters (nodePtr node)
 static void
 feed_remove (nodePtr node)
 {
-	ui_node_remove_node (node);
+	feed_list_node_remove_node (node);
 	
 	favicon_remove_from_cache (node->id);
 	db_subscription_remove (node->id);
@@ -404,7 +403,8 @@ feed_get_node_type (void)
 	static struct nodeType nti = {
 		NODE_CAPABILITY_SHOW_UNREAD_COUNT |
 		NODE_CAPABILITY_UPDATE |
-		NODE_CAPABILITY_UPDATE_FAVICON,
+		NODE_CAPABILITY_UPDATE_FAVICON |
+		NODE_CAPABILITY_EXPORT,
 		"feed",		/* not used, feed format ids are used instead */
 		NULL,
 		feed_import,
