@@ -1,7 +1,7 @@
 /**
  * @file feed_parser.c  parsing of different feed formats
  * 
- * Copyright (C) 2008 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2008-2010 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,6 +67,9 @@ feed_type_str_to_fhp (const gchar *str)
 	
 	if (!str)
 		return NULL;
+		
+	if (strstr(str, "pie"))
+		return feed_type_str_to_fhp ("atom");
 
 	for(iter = feed_parsers_get_list (); iter != NULL; iter = iter->next) {
 		fhp = (feedHandlerPtr)iter->data;
@@ -118,6 +121,7 @@ feed_parser_auto_discover (feedParserCtxtPtr ctxt)
 	
 	source = html_auto_discover_feed (ctxt->data, subscription_get_source (ctxt->subscription));
 
+	/* FIXME: we only need the !g_str_equal as a workaround after a 404 */
 	if (source && !g_str_equal (source, subscription_get_source (ctxt->subscription))) {
 		debug1 (DEBUG_UPDATE, "Discovered link: %s", source);
 		ctxt->failed = FALSE;
@@ -176,14 +180,14 @@ feed_parse (feedParserCtxtPtr ctxt)
 			cur = cur->next;
 		}
 		
+		if(!cur)
+			break;
+				
 		if(!cur->name) {
 			g_string_append(ctxt->feed->parseErrors, _("Invalid XML!"));
 			break;
 		}
-		
-		if(!cur)
-			break;
-		
+
 		/* determine the syndication format and start parser */
 		GSList *handlerIter = feed_parsers_get_list ();
 		while(handlerIter) {
