@@ -1,7 +1,7 @@
 /**
  * @file node_source.c  generic node source provider implementation
  * 
- * Copyright (C) 2005-2010 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2005-2013 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include "common.h"
 #include "db.h"
 #include "debug.h"
+#include "feed.h"
 #include "feedlist.h"
 #include "folder.h"
 #include "item_state.h"
@@ -35,11 +36,15 @@
 #include "ui/icons.h"
 #include "ui/liferea_dialog.h"
 #include "ui/ui_common.h"
-#include "ui/ui_node.h"
+#include "ui/feed_list_node.h"
+#include "fl_sources/aol_source.h"
 #include "fl_sources/default_source.h"
 #include "fl_sources/dummy_source.h"
 #include "fl_sources/google_source.h"
+#include "fl_sources/inoreader_source.h"
 #include "fl_sources/opml_source.h"
+#include "fl_sources/reedah_source.h"
+#include "fl_sources/theoldreader_source.h"
 #include "fl_sources/ttrss_source.h"
 
 static GSList	*nodeSourceTypes = NULL;
@@ -90,11 +95,15 @@ node_source_setup_root (void)
 	debug_enter ("node_source_setup_root");
 	
 	/* we need to register all source types once before doing anything... */
+	//node_source_type_register (aol_source_get_type ());
 	node_source_type_register (default_source_get_type ());
 	node_source_type_register (dummy_source_get_type ());
 	node_source_type_register (opml_source_get_type ());
 	node_source_type_register (google_source_get_type ());
+//	node_source_type_register (inoreader_source_get_type ());
+	node_source_type_register (reedah_source_get_type ());
 	node_source_type_register (ttrss_source_get_type ());
+	node_source_type_register (theoldreader_source_get_type ());
 
 	type = node_source_type_find (NULL, NODE_SOURCE_CAPABILITY_IS_ROOT);
 	if (!type) 
@@ -215,22 +224,23 @@ on_node_source_type_response (GtkDialog *dialog, gint response_id, gpointer user
 	if (response_id == GTK_RESPONSE_OK) {
 		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (liferea_dialog_lookup (GTK_WIDGET (dialog), "type_list")));
 		g_assert (NULL != selection);
-		gtk_tree_selection_get_selected (selection, &model, &iter);
-		gtk_tree_model_get (model, &iter, 1, &type, -1);
-		type->source_new ();
+		if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+			gtk_tree_model_get (model, &iter, 1, &type, -1);
+			if (type)
+				type->source_new ();
+		}
 	}
 	
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 static gboolean
-ui_node_source_type_dialog (void)
+feed_list_node_source_type_dialog (void)
 {
 	GSList 			*iter = nodeSourceTypes;
 	GtkWidget 		*dialog, *treeview;
 	GtkTreeStore		*treestore;
 	GtkCellRenderer		*renderer;
-	GtkTreeViewColumn 	*column;
 	GtkTreeIter		treeiter;
 	nodeSourceTypePtr	type;
 
@@ -262,7 +272,6 @@ ui_node_source_type_dialog (void)
 	treeview = liferea_dialog_lookup (dialog, "type_list");
 	g_assert (NULL != treeview);
 
-	column = gtk_tree_view_column_new ();
 	renderer = gtk_cell_renderer_text_new ();
 	g_object_set (renderer, "wrap-width", 400, NULL);
 	g_object_set (renderer, "wrap-mode", PANGO_WRAP_WORD, NULL);
@@ -358,6 +367,61 @@ node_source_item_set_flag (nodePtr node, itemPtr item, gboolean newState)
 		item_flag_state_changed (item, newState);
 }
 
+static void
+node_source_convert_to_local_child_node (nodePtr node) 
+{
+	/* Ensure to remove special subscription types and cancel updates 
+	   Note: we expect that all feeds already have the subscription URL
+	   set. This might need to be done by the node type specific 
+	   convert_to_local() method! */
+	if (node->subscription) {
+		update_job_cancel_by_owner ((gpointer)node);
+		update_job_cancel_by_owner ((gpointer)node->subscription);
+
+		debug2 (DEBUG_UPDATE, "Converting feed: %s = %s\n", node->title, node->subscription->source);
+
+		node->subscription->type = feed_get_subscription_type ();
+	}
+
+	if (IS_FOLDER (node))
+		node_foreach_child (node, node_source_convert_to_local_child_node);
+
+	node->source = ((nodePtr)feedlist_get_root ())->source;
+}
+
+void
+node_source_convert_to_local (nodePtr node)
+{
+	g_assert (node == node->source->root);
+
+	/* Preparation */
+
+	update_job_cancel_by_owner ((gpointer)node);
+	update_job_cancel_by_owner ((gpointer)node->subscription);
+	update_job_cancel_by_owner ((gpointer)node->source);
+
+	/* Give the node source type the chance to do things ... */
+	if (NULL != NODE_SOURCE_TYPE (node)->convert_to_local)
+		NODE_SOURCE_TYPE (node)->convert_to_local (node);
+
+	/* Perform conversion */
+	
+	debug0 (DEBUG_UPDATE, "Converting root node to folder...");
+	node->source = ((nodePtr)feedlist_get_root ())->source;
+	node->type = folder_get_node_type ();
+	node->subscription = NULL;	/* leaking subscription is ok */
+	node->data = NULL;		/* leaking data is ok */
+
+	node_foreach_child (node, node_source_convert_to_local_child_node);
+
+	feedlist_schedule_save ();
+
+	/* FIXME: something is not perfect, because if you immediately 
+	   remove the subscription tree afterwards there is a double free */
+
+	ui_show_info_box (_("The '%s' subscription was successfully converted to local feeds!"), node->title);
+}
+
 /* implementation of the node type interface */
 
 static void
@@ -368,7 +432,7 @@ node_source_remove (nodePtr node)
 	if (NULL != NODE_SOURCE_TYPE (node)->source_delete)
 		NODE_SOURCE_TYPE (node)->source_delete (node);
 		
-	ui_node_remove_node (node);
+	feed_list_node_remove_node (node);
 }
 
 static void
@@ -411,8 +475,8 @@ node_source_get_node_type (void)
 		nodeType->update_counters	= folder_get_node_type()->update_counters;
 		nodeType->remove		= node_source_remove;
 		nodeType->render		= node_default_render;
-		nodeType->request_add		= ui_node_source_type_dialog;
-		nodeType->request_properties	= ui_node_rename;
+		nodeType->request_add		= feed_list_node_source_type_dialog;
+		nodeType->request_properties	= feed_list_node_rename;
 		nodeType->free			= node_source_free;
 	}
 
