@@ -1,7 +1,7 @@
 /**
  * @file opml_source.c  OPML Planet/Blogroll feed list source
  * 
- * Copyright (C) 2006-2010 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2013 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #include "xml.h"
 #include "ui/icons.h"
 #include "ui/liferea_dialog.h"
+#include "ui/ui_common.h"
 
 /** default OPML update interval = once a day */
 #define OPML_SOURCE_UPDATE_INTERVAL 60*60*24
@@ -228,7 +229,7 @@ static void ui_opml_source_get_source_url (void);
 gchar *
 opml_source_get_feedlist (nodePtr node)
 {
-	return common_create_cache_filename ("cache" G_DIR_SEPARATOR_S "plugins", node->id, "opml");
+	return common_create_cache_filename ("plugins", node->id, "opml");
 }
 
 void
@@ -321,22 +322,28 @@ static void opml_source_deinit(void) { }
 /* node source type definition */
 
 static struct nodeSourceType nst = {
-	"fl_opml",
-	N_("Planet, BlogRoll, OPML"),
-	N_("Integrate blogrolls or Planets in your feed list. Liferea will "
-	   "automatically add and remove feeds according to the changes of "
-	   "the source OPML document"),
-	NODE_SOURCE_CAPABILITY_DYNAMIC_CREATION,
-	opml_source_init,
-	opml_source_deinit,
-	ui_opml_source_get_source_url,
-	opml_source_remove,
-	opml_source_import,
-	opml_source_export,
-	opml_source_get_feedlist,
-	opml_source_update,
-	opml_source_auto_update,
-	NULL	/* free */
+	.id                  = "fl_opml",
+	.name                = N_("Planet, BlogRoll, OPML"),
+	.description         = N_("Integrate blogrolls or Planets in your feed list. Liferea will "
+	                          "automatically add and remove feeds according to the changes of "
+	                          "the source OPML document"),
+	.capabilities        = NODE_SOURCE_CAPABILITY_DYNAMIC_CREATION,
+	.source_type_init    = opml_source_init,
+	.source_type_deinit  = opml_source_deinit,
+	.source_new          = ui_opml_source_get_source_url,
+	.source_delete       = opml_source_remove,
+	.source_import       = opml_source_import,
+	.source_export       = opml_source_export,
+	.source_get_feedlist = opml_source_get_feedlist,
+	.source_update       = opml_source_update,
+	.source_auto_update  = opml_source_auto_update,
+	.free                = NULL,
+	.item_set_flag       = NULL,
+	.item_mark_read      = NULL,
+	.add_folder          = NULL,
+	.add_subscription    = NULL,
+	.remove_node         = NULL,
+	.convert_to_local    = NULL
 };
 
 nodeSourceTypePtr
@@ -384,7 +391,7 @@ on_opml_file_selected (const gchar *filename, gpointer user_data)
 static void
 on_opml_file_choose_clicked (GtkButton *button, gpointer user_data)
 {
-	ui_choose_file (_("Choose OPML File"), GTK_STOCK_OPEN, FALSE, on_opml_file_selected, NULL, NULL, "*.opml", _("OPML Files"), user_data);
+	ui_choose_file (_("Choose OPML File"), GTK_STOCK_OPEN, FALSE, on_opml_file_selected, NULL, NULL, "*.opml|*.xml", _("OPML Files"), user_data);
 }
 
 static void
