@@ -1,7 +1,7 @@
 /**
- * @file itemlist.c  itemlist handling
+ * @file itemlist.c  item list handling
  *
- * Copyright (C) 2004-2012 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2004-2012 Lars Windolf <lars.lindner@gmail.com>
  *	      
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include "feed.h"
 #include "feedlist.h"
 #include "folder.h"
+#include "item_history.h"
 #include "item_state.h"
 #include "itemlist.h"
 #include "itemset.h"
@@ -41,7 +42,7 @@
 #include "ui/liferea_shell.h"
 #include "ui/feed_list_view.h"
 #include "ui/liferea_htmlview.h"
-#include "ui/ui_node.h"
+#include "ui/feed_list_node.h"
 
 /* The 'item list' is a controller for 'item view' and database backend. 
    It manages the currently displayed 'node', realizes filtering
@@ -187,7 +188,7 @@ itemlist_check_for_deferred_action (void)
 			itemlist->priv->deferredFilter = FALSE;
 			item = item_load(id);
 			itemview_remove_item(item);
-			ui_node_update(item->nodeId);
+			feed_list_node_update(item->nodeId);
 		}
 
 		/* check for removals caused by vfolder rules */
@@ -416,14 +417,14 @@ void
 itemlist_toggle_flag (itemPtr item) 
 {
 	item_set_flag_state (item, !(item->flagStatus));
-	itemview_update ();
+	/* No itemview_update() to avoid disturbing HTML scroll state and media content */
 }
 
 void
 itemlist_toggle_read_status (itemPtr item) 
 {
 	item_set_read_state (item, !(item->readStatus));
-	itemview_update ();
+	/* No itemview_update() to avoid disturbing HTML scroll state and media content */
 }
 
 /* function to remove items due to item list filtering */
@@ -434,7 +435,7 @@ itemlist_hide_item (itemPtr item)
 	   don't do it and set a flag to do it when unselecting */
 	if (itemlist->priv->selectedId != item->id) {
 		itemview_remove_item (item);
-		ui_node_update (item->nodeId);
+		feed_list_node_update (item->nodeId);
 	} else {
 		itemlist->priv->deferredFilter = TRUE;
 		/* update the item to show new state that forces
@@ -456,9 +457,6 @@ itemlist_unhide_item (itemPtr item)
 void
 itemlist_remove_item (itemPtr item) 
 {
-	/* update search folder counters */
-	vfolder_foreach_data (vfolder_remove_item, item);
-	
 	if (itemlist->priv->selectedId == item->id) {
 		itemlist_set_selected (NULL);
 		itemlist->priv->deferredFilter = FALSE;
@@ -471,15 +469,16 @@ itemlist_remove_item (itemPtr item)
 	itemview_update ();
 
 	db_item_remove (item->id);
-	
-	/* update feed list */
+
+	/* update feed list counters*/
+	vfolder_foreach (node_update_counters);
 	node_update_counters (node_from_id (item->nodeId));
 	
 	item_unload (item);
 }
 
 /* soft possibly delayed item remove */
-void
+static void
 itemlist_request_remove_item (itemPtr item) 
 {	
 	/* if the currently selected item should be removed we
@@ -502,8 +501,6 @@ itemlist_remove_items (itemSetPtr itemSet, GList *items)
 	while (iter) {
 		itemPtr item = (itemPtr) iter->data;
 
-		vfolder_foreach_data (vfolder_remove_item, item);
-
 		if (itemlist->priv->selectedId != item->id) {
 			/* don't call itemlist_remove_item() here, because it's to slow */
 			itemview_remove_item (item);
@@ -517,27 +514,15 @@ itemlist_remove_items (itemSetPtr itemSet, GList *items)
 	}
 
 	itemview_update ();
+	vfolder_foreach (node_update_counters);
 	node_update_counters (node_from_id (itemSet->nodeId));
 }
 
 void
 itemlist_remove_all_items (nodePtr node)
 {	
-	GList		*iter;
-	itemSetPtr	itemset;
-	
 	if (node == itemlist->priv->currentNode)
 		itemview_clear ();
-
-	itemset = db_itemset_load (node->id);
-	iter = itemset->ids;
-	while (iter) {
-		itemPtr item = item_load ((gulong)iter->data);
-		vfolder_foreach_data (vfolder_remove_item, item);
-		item_unload (item);
-		iter = g_list_next (iter);
-	}
-	itemset_free (itemset);
 		
 	db_itemset_remove_all (node->id);
 	
@@ -546,6 +531,7 @@ itemlist_remove_all_items (nodePtr node)
 		itemlist_duplicate_list_free ();
 	}
 
+	vfolder_foreach (node_update_counters);
 	node_update_counters (node);
 }
 
@@ -559,13 +545,10 @@ itemlist_update_item (itemPtr item)
 		itemlist_unhide_item (item);
 	}
 
-	/* FIXME: this is tricky. It's possible that the item
+	/* FIXME: this is tricky. It's possible that the item is
 	 * selected, but the itemview contains a webpage. In
-	 * that case, we don't want to reload the item, but if
-	 * the itemview contains the item, we want, as the
-	 * background will turn yellow and the the title will
-	 * turn red. So how to know whether the itemview contains
-	 * the item?
+	 * that case, we don't want to reload the item.
+	 * So how to know whether the itemview contains the item?
 	 */
 	itemview_update_item (item);
 }
@@ -582,7 +565,7 @@ itemlist_selection_changed (itemPtr item)
 		   more matching the display rules because they have changed state */
 		itemlist_check_for_deferred_action ();
 
-		debug1 (DEBUG_GUI, "item list selection changed to \"%s\"", item_get_title (item));
+		debug1 (DEBUG_GUI, "item list selection changed to \"%s\"", item?item_get_title (item):"(null)");
 		
 		itemlist_set_selected (item);
 	
@@ -604,7 +587,7 @@ itemlist_selection_changed (itemPtr item)
 				itemview_select_item (item);
 				itemview_update ();
 			}
-			ui_node_update (item->nodeId);
+			feed_list_node_update (item->nodeId);
 		}
 
 		feedlist_reset_new_item_count ();
@@ -661,6 +644,46 @@ on_view_activate (GtkRadioAction *action, GtkRadioAction *current, gpointer user
 	itemlist_set_view_mode (val);
 }
 
+static void
+itemlist_select_from_history (gboolean back)
+{
+	itemPtr item;
+	nodePtr node;
+
+	if (back)
+		item = item_history_get_previous ();
+	else 
+		item = item_history_get_next ();
+
+	if (!item)
+		return;
+
+	node = node_from_id (item->parentNodeId);
+	if (!node)
+		return;
+
+	if (node != feedlist_get_selected ())
+		feed_list_view_select (node);
+
+	if (NODE_VIEW_MODE_COMBINED == itemlist_get_view_mode ())
+		itemlist_set_view_mode (NODE_VIEW_MODE_NORMAL);
+
+	itemview_select_item (item);
+	item_unload (item);
+}
+
+void
+on_prev_read_item_activate (GtkMenuItem *menuitem, gpointer user_data)
+{
+	itemlist_select_from_history (TRUE);
+}
+
+void
+on_next_read_item_activate (GtkMenuItem *menuitem, gpointer user_data)
+{
+	itemlist_select_from_history (FALSE);
+}
+
 /* item loader methods */
 
 static void
@@ -687,7 +710,7 @@ itemlist_item_batch_fetched_cb (ItemLoader *il, GSList *items, gpointer user_dat
 	g_slist_free (items);
 }
 
-void
+static void
 itemlist_add_loader (ItemLoader *loader)
 {
 	g_signal_connect (G_OBJECT (loader), "item-batch-fetched", G_CALLBACK (itemlist_item_batch_fetched_cb), NULL);
