@@ -1,7 +1,7 @@
 /**
  * @file main.c Liferea main program
  *
- * Copyright (C) 2003-2012 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2012 Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *  
  * Some code like the command line handling was inspired by 
@@ -29,12 +29,12 @@
 #endif
 
 #include <gtk/gtk.h>
-#include <unique/unique.h>
 
 #include <string.h>
 #include <stdlib.h>
 #include <signal.h>
 #include <unistd.h>
+#include <girepository.h>
 
 #include "conf.h"
 #include "common.h"
@@ -45,7 +45,6 @@
 #include "social.h"
 #include "update.h"
 #include "xml.h"
-#include "ui/ui_session.h"
 #include "ui/liferea_shell.h"
 #include "notification/notification.h"
 
@@ -55,61 +54,41 @@ static enum {
 	STATE_SHUTDOWN
 } runState = STATE_STARTING;
 
-enum {
-	COMMAND_0 = 0, /* 0 is not a valid command */
-	COMMAND_ADD_FEED
-};
+static const gchar *initialStateOption = NULL;
 
-static UniqueResponse
-message_received_cb (UniqueApp         *app,
-		     UniqueCommand      command,
-		     UniqueMessageData *message,
-		     guint              time_,
-		     gpointer           user_data)
+/* GApplication "open" callback for receiving feed-add requests from remote */
+static void
+on_feed_add (GApplication *application,
+             gpointer      files,
+             gint          n_files,
+             gchar        *hint,
+             gpointer      user_data)
 {
-	UniqueResponse res;
-	gchar *feed;
+	GFile **uris = (GFile **)files;	/* we always expect only one URI */
 
-	debug1(DEBUG_GUI, "libunique command received >>>%d<<<", command);
-
-	switch (command)
-	{
-		case UNIQUE_ACTIVATE:
-			/* Raise the liferea window */
-			debug0 (DEBUG_GUI, "-> raise window requested");
-			liferea_shell_present ();
-			res = UNIQUE_RESPONSE_OK;
-			break;
-		case COMMAND_ADD_FEED:
-			feed = unique_message_data_get_text (message);
-			feedlist_add_subscription (feed, NULL, NULL, 0);
-
-			res = UNIQUE_RESPONSE_OK;
-			break;
-		default:
-			g_warning ("Received unknown libunique command: >>>%d<<<", command);
-			res = UNIQUE_RESPONSE_OK;
-			break;
-	}
-
-	return res;
+	g_assert(uris);
+	g_assert(n_files == 1);
+	feedlist_add_subscription (g_file_get_uri (uris[0]), NULL, NULL, 0);
 }
 
-static void G_GNUC_NORETURN
-fatal_signal_handler (int sig)
+/* GApplication "activate" callback for normal startup and also processing
+   local feed-add requests (URI passed in user_data). */
+static void
+on_app_activate (GtkApplication *app, gpointer user_data)
 {
-	sigset_t sigset;
+	GList		*list;
+	gchar		*feedUri = (gchar *)user_data;
 
-	sigemptyset (&sigset);
-	sigprocmask (SIG_SETMASK, &sigset, NULL);
+	list = gtk_application_get_windows (app);
 
-	g_print ("\nLiferea did receive signal %d (%s).\n", sig, g_strsignal (sig));
+	if (list) {
+		gtk_window_present (GTK_WINDOW (list->data));
+	} else {
+		liferea_shell_create (app, initialStateOption);
 
-	g_print ("You have probably triggered a program bug. I will now try to \n");
-	g_print ("create a backtrace which you can attach to any support requests.\n\n");
-	g_on_error_stack_trace (PACKAGE);
-
-	_exit (1);
+		if (feedUri)
+			feedlist_add_subscription (feedUri, NULL, NULL, 0);
+	}
 }
 
 static void
@@ -172,30 +151,20 @@ show_version (const gchar *option_name,
 int
 main (int argc, char *argv[])
 {
-	UniqueApp	*app;
-	UniqueMessageData	*msg;
+	GtkApplication	*app;
 	GError		*error = NULL;
 	GOptionContext	*context;
 	GOptionGroup	*debug;
 	gulong		debug_flags = 0;
 	LifereaDBus	*dbus = NULL;
-	const gchar	*initial_state = "shown";
-	gchar		*feed = NULL;
-	int		initialState;
-	gboolean	show_tray_icon, start_in_tray;
-
-#ifdef USE_SM
-	gchar *opt_session_arg = NULL;
-#endif
+	gchar		*feedUri = NULL;
+	gint 		status;
 
 	GOptionEntry entries[] = {
-		{ "mainwindow-state", 'w', 0, G_OPTION_ARG_STRING, &initial_state, N_("Start Liferea with its main window in STATE. STATE may be `shown', `iconified', or `hidden'"), N_("STATE") },
-#ifdef USE_SM
-		{ "session", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &opt_session_arg, NULL, NULL },
-#endif
+		{ "mainwindow-state", 'w', 0, G_OPTION_ARG_STRING, &initialStateOption, N_("Start Liferea with its main window in STATE. STATE may be `shown', `iconified', or `hidden'"), N_("STATE") },
 		{ "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, show_version, N_("Show version information and exit"), NULL },
-		{ "add-feed", 'a', 0, G_OPTION_ARG_STRING, &feed, N_("Add a new subscription"), N_("uri") },
-		{ NULL }
+		{ "add-feed", 'a', 0, G_OPTION_ARG_STRING, &feedUri, N_("Add a new subscription"), N_("uri") },
+		{ NULL, 0, 0, 0, NULL, NULL, NULL }
 	};
 
 	GOptionEntry debug_entries[] = {
@@ -204,7 +173,7 @@ main (int argc, char *argv[])
 		{ "debug-conf", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages for the configuration handling"), NULL },
 		{ "debug-db", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of the database handling"), NULL },
 		{ "debug-gui", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of all GUI functions"), NULL },
-		{ "debug-html", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Enables HTML rendering debugging. Each time Liferea renders HTML output it will also dump the generated HTML into ~/.liferea_1.8/output.xhtml"), NULL },
+		{ "debug-html", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Enables HTML rendering debugging. Each time Liferea renders HTML output it will also dump the generated HTML into ~/.cache/liferea/output.xhtml"), NULL },
 		{ "debug-net", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of all network activity"), NULL },
 		{ "debug-parsing", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of all parsing functions"), NULL },
 		{ "debug-performance", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages when a function takes too long to process"), NULL },
@@ -212,11 +181,9 @@ main (int argc, char *argv[])
 		{ "debug-update", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of the feed update processing"), NULL },
 		{ "debug-vfolder", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print debugging messages of the search folder matching"), NULL },
 		{ "debug-verbose", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, debug_entries_parse_callback, N_("Print verbose debugging messages"), NULL },
-		{ NULL }
+		{ NULL, 0, 0, 0, NULL, NULL, NULL }
 	};
 
-	if (!g_thread_supported ()) g_thread_init (NULL);
-
 #ifdef ENABLE_NLS
 	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -233,11 +200,12 @@ main (int argc, char *argv[])
 
 	context = g_option_context_new (NULL);
 	g_option_context_set_summary (context, N_("Liferea, the Linux Feed Reader"));
-	g_option_context_set_description (context, N_("For more information, please visit http://liferea.sourceforge.net/"));
+	g_option_context_set_description (context, N_("For more information, please visit http://lzone.de/liferea/"));
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
 	g_option_context_set_translation_domain(context, GETTEXT_PACKAGE);
 	g_option_context_add_group (context, debug);
 	g_option_context_add_group (context, gtk_get_option_group (FALSE));
+	g_option_context_add_group (context, g_irepository_get_option_group ());
 
 	g_option_context_parse (context, &argc, &argv, &error);
 	g_option_context_free (context);
@@ -257,26 +225,15 @@ main (int argc, char *argv[])
 
 	gtk_init (&argc, &argv);
 
-	/* Single instance checks */
-	app = unique_app_new_with_commands ("net.sourceforge.liferea", NULL,
-					    "add_feed", COMMAND_ADD_FEED,
-					    NULL);
-	if (unique_app_is_running (app)) {
-		g_print ("Liferea is already running\n");
-		unique_app_send_message (app, UNIQUE_ACTIVATE, NULL);
-		if (feed) {
-			msg = unique_message_data_new ();
-			unique_message_data_set_text (msg, feed, -1);
-			unique_app_send_message (app, COMMAND_ADD_FEED, msg);
-		}
-		return 1;
-	} else {
-		g_signal_connect (app, "message-received", G_CALLBACK (message_received_cb), NULL);
-	}
+	/* Single instance checks, also note that we pass or only RPC (add-feed)
+	   as activate signal payload as it is simply an URI string. */
+	app = gtk_application_new ("net.sourceforge.liferea", G_APPLICATION_HANDLES_OPEN);
+	g_signal_connect (app, "activate", G_CALLBACK (on_app_activate), feedUri);
+	g_signal_connect (app, "open", G_CALLBACK (on_feed_add), NULL);
 
-	/* GTK theme support */
+	g_set_prgname ("liferea");
 	g_set_application_name (_("Liferea"));
-	gtk_window_set_default_icon_name ("liferea");
+	gtk_window_set_default_icon_name ("liferea");	/* GTK theme support */
 
 	debug_start_measurement (DEBUG_DB);
 
@@ -290,53 +247,34 @@ main (int argc, char *argv[])
 
 	dbus = liferea_dbus_new ();
 
-	/* how to start liferea, command line takes precedence over preferences */
-	conf_get_bool_value (SHOW_TRAY_ICON, &show_tray_icon);
-	conf_get_bool_value (START_IN_TRAY, &start_in_tray);
-	if (g_str_equal(initial_state, "iconified")) {
-		initialState = MAINWINDOW_ICONIFIED;
-	} else if (g_str_equal(initial_state, "hidden") ||
-	    (show_tray_icon && start_in_tray)) {
-		initialState = MAINWINDOW_HIDDEN;
-	} else {
-		initialState = MAINWINDOW_SHOWN;
-	}
-
-	liferea_shell_create (initialState);
-	g_set_prgname ("liferea");
-	
-#ifdef USE_SM
-	/* This must be after feedlist reading because some session
-	   managers will tell Liferea to exit if Liferea does not
-	   respond to SM requests within a minute or two. This starts
-	   the main loop soon after opening the SM connection. */
-	session_init (BIN_DIR G_DIR_SEPARATOR_S "liferea", opt_session_arg);
-	session_set_cmd (NULL, initialState);
-#endif
 	signal (SIGTERM, signal_handler);
 	signal (SIGINT, signal_handler);
 	signal (SIGHUP, signal_handler);
 
-#ifndef G_OS_WIN32
-	signal (SIGBUS, fatal_signal_handler);
-	signal (SIGSEGV, fatal_signal_handler);
-#endif
-
 	/* Note: we explicitely do not use the gdk_thread_*
 	   locking in Liferea because it freezes the program
-	   when running Flash applets in gtkmozembed */
+	   when running Flash applets */
 
 	runState = STATE_STARTING;
 	
 	debug_end_measurement (DEBUG_DB, "startup");
 
-	if (feed)
-		feedlist_add_subscription (feed, NULL, NULL, 0);
+	status = g_application_run (G_APPLICATION (app), 0, NULL);
 
-	gtk_main ();
+	/* Trigger RPCs if we are not primary instance (currently only feed-add) */
+	if (feedUri && g_application_get_is_remote (G_APPLICATION (app))) {
+		GFile *uris[2];
+
+		uris[0] = g_file_new_for_uri (feedUri);
+		uris[1] = NULL;
+		g_application_open (G_APPLICATION (app), uris, 1, "feed-add");
+		g_object_unref (uris[0]);
+	}
 	
 	g_object_unref (G_OBJECT (dbus));
-	return 0;
+	g_object_unref (app);
+
+	return status;
 }
 
 static gboolean
@@ -352,18 +290,13 @@ on_shutdown (gpointer user_data)
 
 	/* order is important ! */
 	update_deinit ();
-	db_deinit ();
-	social_free ();
 
 	liferea_shell_destroy ();
-#ifdef USE_SM
-	/* unplug */
-	session_end ();
-#endif
+
+	db_deinit ();
+	social_free ();
 	conf_deinit ();
 	
-	gtk_main_quit ();
-	
 	debug_exit ("liferea_shutdown");
 	return FALSE;
 }
