@@ -2,7 +2,8 @@
  * @file metadata.c  handling of typed item and feed meta data
  *
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
- * Copyright (C) 2004-2010 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2004-2010 Lars Windolf <lars.lindner@gmail.com>
+ * Copyright (C) 2015 Rich Coe <rcoe@wi.rr.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -147,6 +148,15 @@ metadata_get_type (const gchar *name)
 	return type;
 }
 
+static gint
+metadata_value_cmp (gconstpointer a, gconstpointer b)
+{
+	if (g_str_equal ((gchar *)a, (gchar *)b))
+		return 0;
+
+	return 1;
+}
+
 GSList *
 metadata_list_append (GSList *metadata, const gchar *strid, const gchar *data)
 {
@@ -194,7 +204,11 @@ metadata_list_append (GSList *metadata, const gchar *strid, const gchar *data)
 	while (iter) {
 		p = (struct pair*)iter->data; 
 		if (g_str_equal (p->strid, strid)) {
-			p->data = g_slist_append (p->data, checked_data);
+			/* Avoid duplicate values */
+			if (NULL == g_slist_find_custom (p->data, checked_data, metadata_value_cmp))
+				p->data = g_slist_append (p->data, checked_data);
+                        else
+                                g_free (checked_data);
 			return metadata;
 		}
 		iter = iter->next;
@@ -215,13 +229,8 @@ metadata_list_set (GSList **metadata, const gchar *strid, const gchar *data)
 	while (iter) {
 		p = (struct pair*)iter->data; 
 		if (g_str_equal (p->strid, strid)) {
-			if (p->data) {
-				/* exchange old value */
-				g_free (((GSList *)p->data)->data);
-				((GSList *)p->data)->data = g_strdup (data);
-			} else {
-				p->data = g_slist_append (p->data, g_strdup (data));
-			}
+                        g_slist_free_full (p->data, g_free);
+                        p->data = g_slist_append (NULL, g_strdup (data));
 			return;
 		}
 		iter = iter->next;
@@ -297,18 +306,12 @@ metadata_list_copy (GSList *list)
 void
 metadata_list_free (GSList *metadata)
 {
-	GSList		*list2, *iter2, *iter = metadata;
+	GSList		*iter = metadata;
 	struct pair	*p;
 	
 	while (iter) {
 		p = (struct pair*)iter->data;
-		list2 = p->data;
-		iter2 = list2;
-		while (iter2) {
-			g_free (iter2->data);
-			iter2 = iter2->next;
-		}
-		g_slist_free (list2);
+                g_slist_free_full (p->data, g_free);
 		g_free (p->strid);
 		g_free (p);
 		iter = iter->next;
@@ -335,26 +338,3 @@ metadata_add_xml_nodes (GSList *metadata, xmlNodePtr parentNode)
 	}
 }
 
-GSList *
-metadata_parse_xml_nodes (xmlNodePtr cur)
-{
-	xmlNodePtr	attribute = cur->xmlChildrenNode;
-	GSList 		*metadata = NULL;
-	
-	while (attribute) {
-		if (attribute->type == XML_ELEMENT_NODE &&
-		    !xmlStrcmp (attribute->name, BAD_CAST"attribute")) {
-			xmlChar *name = xmlGetProp (attribute, BAD_CAST"name");
-			if (name) {
-				gchar *value = xmlNodeListGetString (cur->doc, attribute->xmlChildrenNode, TRUE);
-				if (value) {
-					metadata = metadata_list_append (metadata, name, value);
-					xmlFree (value);
-				}
-				xmlFree (name);
-			}
-		}
-		attribute = attribute->next;
-	}
-	return metadata;
-}
