@@ -1,7 +1,7 @@
 /**
  * @file net.c  HTTP network access using libsoup
  *
- * Copyright (C) 2007-2009 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2014 Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2009 Emilio Pozuelo Monfort <pochu27@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,25 +29,17 @@
 #include <time.h>
 
 #include "common.h"
-#include "conf.h"
 #include "debug.h"
-#include "ui/liferea_htmlview.h"
-#include "ui/liferea_shell.h"
 
 #define HOMEPAGE	"http://liferea.sf.net/"
 
-static SoupSession *session;
-static SoupSession *session_no_proxy;
-static SoupSession *session_no_cookies;
-static SoupSession *session_no_cookies_no_proxy;
+static SoupSession *session = NULL;
 
 static gchar	*proxyname = NULL;
 static gchar	*proxyusername = NULL;
 static gchar	*proxypassword = NULL;
 static int	proxyport = 0;
 
-static gboolean	online = FALSE;
-
 static void
 network_process_callback (SoupSession *session, SoupMessage *msg, gpointer user_data)
 {
@@ -65,7 +57,7 @@ network_process_callback (SoupSession *session, SoupMessage *msg, gpointer user_
 	}
 
 	debug1 (DEBUG_NET, "download status code: %d", msg->status_code);
-	debug1 (DEBUG_NET, "source after download: >>>%s<<<\n", job->result->source);
+	debug1 (DEBUG_NET, "source after download: >>>%s<<<", job->result->source);
 
 	job->result->data = g_memdup (msg->response_body->data, msg->response_body->length+1);
 	job->result->size = (size_t)msg->response_body->length;
@@ -85,6 +77,12 @@ network_process_callback (SoupSession *session, SoupMessage *msg, gpointer user_
 		}
 	}
 
+	/* Update ETag value */
+	tmp = soup_message_headers_get_one (msg->response_headers, "ETag");
+	if (tmp) {
+		job->result->updateState->etag = g_strdup(tmp);
+	}    
+
 	update_process_finished_job (job);
 }
 
@@ -118,11 +116,11 @@ network_process_request (const updateJobPtr const job)
 {
 	SoupMessage	*msg;
 	SoupDate	*date;
-	gboolean	no_proxy = FALSE;
-	gboolean	no_cookies = FALSE;
-	
+
 	g_assert (NULL != job->request);
 	debug1 (DEBUG_NET, "downloading %s", job->request->source);
+	if (job->request->postdata && (debug_level & DEBUG_VERBOSE) && (debug_level & DEBUG_NET))
+		debug1 (DEBUG_NET, "   postdata=>>>%s<<<", job->request->postdata);
 
 	/* Prepare the SoupMessage */
 	msg = soup_message_new (job->request->postdata ? SOUP_METHOD_POST : SOUP_METHOD_GET,
@@ -143,10 +141,10 @@ network_process_request (const updateJobPtr const job)
 	}
 
 	/* Set the If-Modified-Since: header */
-	if (job->request->updateState && job->request->updateState->lastModified) {
+	if (job->request->updateState && update_state_get_lastmodified(job->request->updateState)) {
 		gchar *datestr;
 
-		date = soup_date_new_from_time_t (job->request->updateState->lastModified);
+		date = soup_date_new_from_time_t (update_state_get_lastmodified (job->request->updateState));
 		datestr = soup_date_to_string (date, SOUP_DATE_HTTP);
 		soup_message_headers_append (msg->request_headers,
 					     "If-Modified-Since",
@@ -155,13 +153,23 @@ network_process_request (const updateJobPtr const job)
 		soup_date_free (date);
 	}
 
+	/* Set the If-None-Match: header */
+	if (job->request->updateState && update_state_get_etag (job->request->updateState)) {
+		soup_message_headers_append(msg->request_headers,
+					    "If-None-Match",
+					    update_state_get_etag (job->request->updateState));
+	}
+
+	/* Support HTTP content negotiation */
+	soup_message_headers_append(msg->request_headers, "Accept", "application/atom+xml,application/xml;q=0.9,text/xml;q=0.8,*/*;q=0.7");
+
 	/* Set the authentication */
 	if (!job->request->authValue &&
 	    job->request->options &&
 	    job->request->options->username &&
 	    job->request->options->password) {
 		SoupURI *uri = soup_message_get_uri (msg);
-
+		
 		soup_uri_set_user (uri, job->request->options->username);
 		soup_uri_set_password (uri, job->request->options->password);
 	}
@@ -175,11 +183,7 @@ network_process_request (const updateJobPtr const job)
 	if (job->request->updateState && job->request->updateState->cookies) {
 		soup_message_headers_append (msg->request_headers, "Cookie",
 		                             job->request->updateState->cookies);
-						
-		/* This might be confusing: But what we mean by setting this flag
-		   is that we do want to ignore the cookie file in ~/.liferea_1.x
-		   and pass a single cookie in the headers instead. */			
-		no_cookies = TRUE;
+		soup_message_disable_feature (msg, SOUP_TYPE_COOKIE_JAR);
 	}
 
 	/* TODO: Right now we send the msg, and if it requires authentication and
@@ -190,22 +194,11 @@ network_process_request (const updateJobPtr const job)
 	 * msg to a callback in case of 401 (see soup_message_add_status_code_handler())
 	 * displaying the dialog ourselves, and requeing the msg if we get credentials */
 
-	/* We queue the message in one session or the other depending on the global
-	 * proxy settings and whether the feed properties has the "dont use a proxy" 
-	 * checkbox enabled */
-	if ((job->request->options && job->request->options->dontUseProxy) ||
-	    (NULL == network_get_proxy_host ()))
-		no_proxy = TRUE;
+	/* If the feed has "dont use a proxy" selected, disable the proxy for the msg */
+	if (job->request->options && job->request->options->dontUseProxy)
+		soup_message_disable_feature (msg, SOUP_TYPE_PROXY_URI_RESOLVER);
 
-	if (no_proxy && no_cookies) {
-		soup_session_queue_message (session_no_cookies_no_proxy, msg, network_process_callback, job);
-	} else if (no_proxy) {
-		soup_session_queue_message (session_no_proxy, msg, network_process_callback, job);
-	} else if (no_cookies) {
-		soup_session_queue_message (session_no_cookies, msg, network_process_callback, job);
-	} else {
-		soup_session_queue_message (session, msg, network_process_callback, job);
-	}
+	soup_session_queue_message (session, msg, network_process_callback, job);
 }
 
 static void
@@ -223,7 +216,6 @@ network_authenticate (
 	// FIXME: Handle HTTP 401 too
 }
 
-
 void
 network_init (void)
 {
@@ -235,53 +227,40 @@ network_init (void)
 
 	/* Set an appropriate user agent */
 	if (g_getenv ("LANG")) {
-		/* e.g. "Liferea/1.6.0 (Linux; de_DE; http://liferea.sf.net/)" */
-		useragent = g_strdup_printf ("Liferea/%s (%s; %s; %s)", VERSION, OSNAME, g_getenv ("LANG"), HOMEPAGE);
+		/* e.g. "Liferea/1.10.0 (Linux; de_DE; http://liferea.sf.net/) AppleWebKit (KHTML, like Gecko)" */
+		useragent = g_strdup_printf ("Liferea/%s (%s; %s; %s) AppleWebKit (KHTML, like Gecko)", VERSION, OSNAME, g_getenv ("LANG"), HOMEPAGE);
 	} else {
-		/* e.g. "Liferea/1.6.0 (Linux; http://liferea.sf.net/)" */
-		useragent = g_strdup_printf ("Liferea/%s (%s; %s)", VERSION, OSNAME, HOMEPAGE);
+		/* e.g. "Liferea/1.10.0 (Linux; http://liferea.sf.net/) AppleWebKit (KHTML, like Gecko)" */
+		useragent = g_strdup_printf ("Liferea/%s (%s; %s) AppleWebKit (KHTML, like Gecko)", VERSION, OSNAME, HOMEPAGE);
 	}
 
 	/* Cookies */
-	filename = common_create_cache_filename ("", "cookies", "txt");
+	filename = common_create_config_filename ("cookies.txt");
 	cookies = soup_cookie_jar_text_new (filename, FALSE);
 	g_free (filename);
 
 	/* Initialize libsoup */
 	proxy = network_get_proxy_uri ();
 	session = soup_session_async_new_with_options (SOUP_SESSION_USER_AGENT, useragent,
-	                                               SOUP_SESSION_TIMEOUT, 120,
+						       SOUP_SESSION_TIMEOUT, 120,
 						       SOUP_SESSION_IDLE_TIMEOUT, 30,
-						       SOUP_SESSION_PROXY_URI, proxy,
 						       SOUP_SESSION_ADD_FEATURE, cookies,
+	                                               SOUP_SESSION_ADD_FEATURE_BY_TYPE, SOUP_TYPE_CONTENT_DECODER,
 						       NULL);
-	if (proxy)
+
+	if (proxy) {
+		debug1 (DEBUG_NET, "Initializing libsoup with proxy '%s'", proxy);
+		g_object_set (G_OBJECT (session),
+			      SOUP_SESSION_PROXY_URI, proxy,
+			      NULL);
 		soup_uri_free (proxy);
+	} else {
+		debug0 (DEBUG_NET, "Initializing libsoup with libproxy defaults");
+		soup_session_add_feature_by_type (session, SOUP_TYPE_PROXY_RESOLVER_DEFAULT);
+	}
 		
 	g_signal_connect (session, "authenticate", G_CALLBACK (network_authenticate), NULL);
 
-	/* This session is for those cases where we are told not to use the proxy */
-	session_no_proxy = soup_session_async_new_with_options (SOUP_SESSION_USER_AGENT, useragent,
-	                                                        SOUP_SESSION_TIMEOUT, 120,
-								SOUP_SESSION_IDLE_TIMEOUT, 30,
-								SOUP_SESSION_ADD_FEATURE, cookies,
-								NULL);
-
-	/* This session is for those cases where we need to add specific cookies, e.g. Google Reader.
-	 * Once GNOME #574571 is fixed, we will be able to use the normal session */
-	session_no_cookies = soup_session_async_new_with_options (SOUP_SESSION_USER_AGENT, useragent,
-	                                                          SOUP_SESSION_TIMEOUT, 120,
-								  SOUP_SESSION_IDLE_TIMEOUT, 30,
-								  SOUP_SESSION_PROXY_URI, proxy,
-								  NULL);
-
-	/* And this one is for cases where we need to use our own cookies, and bypass the proxy, e.g.
-	 * Google Reader subscription where the "ignore proxy" preference is set */
-	session_no_cookies_no_proxy = soup_session_async_new_with_options (SOUP_SESSION_USER_AGENT, useragent,
-	                                                                   SOUP_SESSION_TIMEOUT, 120,
-									   SOUP_SESSION_IDLE_TIMEOUT, 30,
-									   NULL);
-
 	/* Soup debugging */
 	if (debug_level & DEBUG_NET) {
 		logger = soup_logger_new (SOUP_LOGGER_LOG_HEADERS, -1);
@@ -323,6 +302,8 @@ network_get_proxy_password (void)
 	return proxypassword;
 }
 
+extern void network_monitor_proxy_changed (void);
+
 void
 network_set_proxy (gchar *host, guint port, gchar *user, gchar *password)
 {
@@ -340,10 +321,7 @@ network_set_proxy (gchar *host, guint port, gchar *user, gchar *password)
 	if (session) {
 		SoupURI *newproxy = network_get_proxy_uri ();
 		
-		g_object_set (session,
-			      SOUP_SESSION_PROXY_URI, newproxy,
-			      NULL);
-		g_object_set (session_no_cookies,
+		g_object_set (G_OBJECT (session),
 			      SOUP_SESSION_PROXY_URI, newproxy,
 			      NULL);
 
@@ -352,8 +330,8 @@ network_set_proxy (gchar *host, guint port, gchar *user, gchar *password)
 	}
 
 	debug4 (DEBUG_NET, "proxy set to http://%s:%s@%s:%d", user, password, host, port);
-
-	liferea_htmlview_update_proxy ();
+	
+	network_monitor_proxy_changed ();
 }
 
 const char *
@@ -406,18 +384,3 @@ network_strerror (gint netstatus, gint httpstatus)
 	return tmp;
 }
 
-void
-network_set_online (gboolean mode)
-{
-	if (online != mode) {
-		online = mode;
-		debug1 (DEBUG_NET, "Changing online mode to %s", mode?"online":"offline");
-		liferea_shell_online_status_changed (mode);
-	}
-}
-
-gboolean
-network_is_online (void)
-{
-	return online;
-}
