@@ -1,7 +1,7 @@
 /**
  * @file node.c  hierarchic feed list node handling
  * 
- * Copyright (C) 2003-2011 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2012 Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #include <string.h>
 
 #include "common.h"
+#include "conf.h"
 #include "db.h"
 #include "debug.h"
 #include "itemlist.h"
@@ -34,7 +35,7 @@
 #include "vfolder.h"
 #include "fl_sources/node_source.h"
 #include "ui/liferea_shell.h"
-#include "ui/ui_node.h"
+#include "ui/feed_list_node.h"
 #include "ui/ui_tray.h"
 
 static GHashTable *nodes = NULL;	/**< node id -> node lookup table */
@@ -87,6 +88,7 @@ node_new (nodeTypePtr type)
 
 	node = (nodePtr)g_new0 (struct node, 1);
 	node->type = type;
+	node->viewMode = NODE_VIEW_MODE_DEFAULT;
 	node->sortColumn = NODE_VIEW_SORT_BY_TIME;
 	node->sortReversed = TRUE;	/* default sorting is newest date at top */
 	node->available = TRUE;
@@ -119,7 +121,7 @@ node_set_subscription (nodePtr node, subscriptionPtr subscription)
 	
 	/* Besides the favicon age we have no persistent 
 	   update state field, so everything else goes NULL */
-	if (node->iconFile) {
+	if (node->iconFile && !strstr(node->iconFile, "default.png")) {
 		subscription->updateState->lastFaviconPoll.tv_sec = common_get_mod_time (node->iconFile);
 		debug2 (DEBUG_UPDATE, "Setting last favicon poll time for %s to %lu", node->id, subscription->updateState->lastFaviconPoll.tv_sec);
 	}
@@ -223,7 +225,7 @@ node_update_parent_counters (nodePtr node)
 	NODE_TYPE (node)->update_counters (node);
 	
 	if (old != node->unreadCount) {
-		ui_node_update (node->id);
+		feed_list_node_update (node->id);
 		ui_tray_update ();
 		liferea_shell_update_unread_stats ();
 	}
@@ -243,7 +245,7 @@ node_update_counters (nodePtr node)
 	
 	if ((oldUnreadCount != node->unreadCount) ||
 	    (oldItemCount != node->itemCount))
-		ui_node_update (node->id);
+		feed_list_node_update (node->id);
 		
 	/* Update the unread count of the parent nodes,
 	   usually they just add all child unread counters */
@@ -276,7 +278,7 @@ node_mark_all_read (nodePtr node)
 	if (!node)
 		return;
 
-	if (0 != node->unreadCount) {
+	if ((node->unreadCount > 0) || (IS_VFOLDER (node))) {
 		itemset_mark_read (node);
 		node->unreadCount = 0;
 		node->needsUpdate = TRUE;
@@ -325,8 +327,8 @@ node_reparent (nodePtr node, nodePtr new_parent)
 	new_parent->children = g_slist_insert (new_parent->children, node, -1);
 	node->parent = new_parent;
 	
-	ui_node_remove_node (node);
-	ui_node_add (node);
+	feed_list_node_remove_node (node);
+	feed_list_node_add (node);
 }
 
 void
@@ -410,7 +412,7 @@ node_set_icon (nodePtr node, gpointer icon)
 	g_free (node->iconFile);
 	
 	if (node->icon)
-		node->iconFile = common_create_cache_filename ("cache" G_DIR_SEPARATOR_S "favicons", node->id, "png");
+		node->iconFile = common_create_cache_filename ("favicons", node->id, "png");
 	else
 		node->iconFile = g_build_filename (PACKAGE_DATA_DIR, PACKAGE, "pixmaps", "default.png", NULL);
 }
@@ -468,13 +470,40 @@ node_set_sort_column (nodePtr node, nodeViewSortType sortColumn, gboolean revers
 void
 node_set_view_mode (nodePtr node, nodeViewType viewMode)
 {
-	node->viewMode = viewMode;
+	gint	defaultViewMode;
+
+	/* To allow users to select a default viewing mode for the layout
+	   we need to store only exceptions from this mode, which is why
+	   we compare the mode to be set with the default and if it's equal
+	   we just set NODE_VIEW_MODE_DEFAULT.
+
+	   This allows to not OPML export the viewMode attribute for nodes
+	   the are in default viewing mode, which then allows to follow
+	   a switch in the preference to a new default viewing mode.
+
+	   This of course also means that the we use some state on each
+	   changing of the view mode preference.
+        */
+
+	conf_get_int_value (DEFAULT_VIEW_MODE, &defaultViewMode);
+
+	if (viewMode != (nodeViewType)defaultViewMode)
+		node->viewMode = viewMode;
+	else
+		node->viewMode = NODE_VIEW_MODE_DEFAULT;
 }
 
 nodeViewType
 node_get_view_mode (nodePtr node)
 {
-	return node->viewMode;
+	gint	defaultViewMode;
+
+	conf_get_int_value (DEFAULT_VIEW_MODE, &defaultViewMode);
+
+	if (NODE_VIEW_MODE_DEFAULT == node->viewMode)
+		return defaultViewMode;
+	else
+		return node->viewMode;
 }
 
 const gchar *
@@ -494,6 +523,28 @@ node_get_base_url(nodePtr node)
 	return baseUrl;
 }
 
+gboolean
+node_can_add_child_feed (nodePtr node)
+{
+	g_assert (node->source->root);
+
+	if (!(NODE_TYPE (node->source->root)->capabilities & NODE_CAPABILITY_ADD_CHILDS))
+		return FALSE;
+
+	return (NODE_SOURCE_TYPE (node)->capabilities & NODE_SOURCE_CAPABILITY_ADD_FEED);
+}
+
+gboolean
+node_can_add_child_folder (nodePtr node)
+{
+	g_assert (node->source->root);
+
+	if (!(NODE_TYPE (node->source->root)->capabilities & NODE_CAPABILITY_ADD_CHILDS))
+		return FALSE;
+
+	return (NODE_SOURCE_TYPE (node)->capabilities & NODE_SOURCE_CAPABILITY_ADD_FOLDER);
+}
+
 /* node children iterating interface */
 
 void
