@@ -2,7 +2,7 @@
  * @file notification.h  generic notification interface
  * 
  * Copyright (C) 2006 Norman Jonas <liferea.sf.net@devport.codepilot.net>
- * Copyright (C) 2006-2008 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2008 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,7 +66,6 @@ typedef struct notificationPlugin {
 
 extern struct notificationPlugin libnotify_plugin;
 
-#ifdef HAVE_LIBNOTIFY
 /**
  * "New items" event callback.
  *
@@ -75,10 +74,6 @@ extern struct notificationPlugin libnotify_plugin;
  * 			regardless of global preference
  */
 void notification_node_has_new_items (nodePtr node, gboolean enforced);
-#else
-static inline void notification_node_has_new_items (nodePtr node, gboolean enforced) {};
-#endif
-
 
 void notification_plugin_register (notificationPluginPtr plugin);
 
