@@ -2,7 +2,7 @@
  * @file atom10.c  Atom 1.0 Parser
  * 
  * Copyright (C) 2005-2006 Nathan Conrad <t98502@users.sourceforge.net>
- * Copyright (C) 2003-2010 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2010 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -234,7 +234,7 @@ atom10_parse_person_construct (xmlNodePtr cur)
 			}
 			
 			if (xmlStrEqual(cur->name, BAD_CAST"uri")) {
-				if (!uri)
+				if (uri)
 					invalid = TRUE;
 				g_free (uri);
 				tmp = (gchar *)xmlNodeListGetString (cur->doc, cur->xmlChildrenNode, 1);
@@ -250,8 +250,12 @@ atom10_parse_person_construct (xmlNodePtr cur)
 		invalid = TRUE;
 		name = g_strdup (_("Invalid Atom feed: unknown author"));
 	}
-	/* FIXME: so something with "invalid" flag */
-	tmp = g_strdup_printf ("%s%s%s", name, uri?uri:"", email?email:"");
+
+	if (!invalid)
+		tmp = g_strdup_printf ("%s%s%s", name, uri?uri:"", email?email:"");
+	else
+		tmp = NULL;
+
 	g_free (uri);
 	g_free (email);
 	g_free (name);
@@ -555,8 +559,10 @@ atom10_parse_feed_author (xmlNodePtr cur, feedParserCtxtPtr ctxt, itemPtr ip, st
 {
 	/* parse feed author */
 	gchar *author = atom10_parse_person_construct (cur);
-	ctxt->subscription->metadata = metadata_list_append (ctxt->subscription->metadata, "author", author);
-	g_free (author);
+	if (author) {
+		ctxt->subscription->metadata = metadata_list_append (ctxt->subscription->metadata, "author", author);
+		g_free (author);
+	}
 	/* FIXME: make item parsing use this author if not specified elsewhere */
 }
 
@@ -582,8 +588,10 @@ atom10_parse_feed_contributor (xmlNodePtr cur, feedParserCtxtPtr ctxt, itemPtr i
 {
 	/* parse feed contributors */
 	gchar *contributer = atom10_parse_person_construct (cur);
-	ctxt->subscription->metadata = metadata_list_append (ctxt->subscription->metadata, "contributor", contributer);
-	g_free (contributer);
+	if (contributer) {
+		ctxt->subscription->metadata = metadata_list_append (ctxt->subscription->metadata, "contributor", contributer);
+		g_free (contributer);
+	}
 }
 
 static void
