@@ -1,7 +1,7 @@
 /**
- * @file rss_item.h RSS item tag parsing 
+ * @file rss_item.h  RSS item tag parsing 
  *
- * Copyright (C) 2003-2006 Lars Lindner <lars.lindner@gmx.net>
+ * Copyright (C) 2003-2008 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +21,8 @@
 #ifndef _RSS_ITEM_H
 #define _RSS_ITEM_H
 
-#include <libxml/tree.h>
-
 #include "item.h"
-#include "feed.h"
+#include "feed_parser.h"
 
 itemPtr parseRSSItem(feedParserCtxtPtr ctxt, xmlNodePtr cur);
 
