@@ -1,7 +1,7 @@
 /**
- * @file render.c  generic XSLT rendering handling
+ * @file render.c  generic GTK theme and XSLT rendering handling
  * 
- * Copyright (C) 2006-2011 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2013 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,13 +35,10 @@
 #include "common.h"
 #include "debug.h"
 #include "item.h"
-#include "itemlist.h"
 #include "itemset.h"
 #include "render.h"
 #include "xml.h"
 #include "ui/liferea_htmlview.h"
-#include "ui/itemview.h"
-#include "ui/liferea_shell.h"
 
 /* Liferea provides special screens and the item and the feed displays
    using self-generated HTML. To separate code and layout and to easily
@@ -61,6 +58,13 @@ static renderParamPtr	langParams = NULL;	/* the current locale settings (for loc
 
 static GHashTable	*stylesheets = NULL;	/* XSLT stylesheet cache */
 
+static void
+render_parameter_free (renderParamPtr paramSet)
+{
+	g_strfreev (paramSet->params);
+	g_free (paramSet);
+}
+
 static void
 render_init (void)
 {
@@ -72,7 +76,7 @@ render_init (void)
 		render_parameter_free (langParams);
 
 	/* Install default stylesheet if it does not yet exist */
-	filename = common_create_cache_filename ("", "liferea", "css");
+	filename = common_create_config_filename ("liferea.css");
 	if (!g_file_test (filename, G_FILE_TEST_EXISTS))
 		common_copy_file (PACKAGE_DATA_DIR "/" PACKAGE "/css/user.css", filename);
 	g_free(filename);
@@ -157,10 +161,11 @@ typedef struct themeColor {
 } *themeColorPtr;
 
 static GSList *themeColors = NULL;
+static gboolean darkTheme = FALSE;
 
 /* Determining of theme colors, to be inserted in CSS */
 static themeColorPtr
-render_get_theme_color (const gchar *name, GdkColor themeColor)
+render_calculate_theme_color (const gchar *name, GdkColor themeColor)
 {
 	themeColorPtr	tc;
 	gushort		r, g, b;
@@ -177,38 +182,95 @@ render_get_theme_color (const gchar *name, GdkColor themeColor)
 	return tc;
 }
 
-static void
-render_get_theme_colors (void)
+static gint
+render_get_rgb_distance (GdkColor *c1, GdkColor *c2)
+{
+	return abs(
+		(299 * c1->red/256 +
+		 587 * c1->green/256 +
+		 114 * c1->blue/256) -
+		(299 * c2->red/256 +
+		 587 * c2->green/256 +
+		 114 * c2->blue/256)
+	       ) / 1000;
+}
+
+void
+render_init_theme_colors (GtkWidget *widget)
 {
 	GtkStyle	*style;
 	GdkColor	*color;
+	gint		textAvg, bgAvg;
 
-	style = itemview_get_style ();
+	style = gtk_widget_get_style (widget);
 
 	g_assert (NULL == themeColors);
-	themeColors = g_slist_append (themeColors, render_get_theme_color ("GTK-COLOR-FG",    style->fg[GTK_STATE_NORMAL]));
-	themeColors = g_slist_append (themeColors, render_get_theme_color ("GTK-COLOR-BG",    style->bg[GTK_STATE_NORMAL]));
-	themeColors = g_slist_append (themeColors, render_get_theme_color ("GTK-COLOR-LIGHT", style->light[GTK_STATE_NORMAL]));
-	themeColors = g_slist_append (themeColors, render_get_theme_color ("GTK-COLOR-DARK",  style->dark[GTK_STATE_NORMAL]));
-	themeColors = g_slist_append (themeColors, render_get_theme_color ("GTK-COLOR-MID",   style->mid[GTK_STATE_NORMAL]));
-	themeColors = g_slist_append (themeColors, render_get_theme_color ("GTK-COLOR-BASE",  style->base[GTK_STATE_NORMAL]));
-	themeColors = g_slist_append (themeColors, render_get_theme_color ("GTK-COLOR-TEXT",  style->text[GTK_STATE_NORMAL]));
+	themeColors = g_slist_append (themeColors, render_calculate_theme_color ("GTK-COLOR-FG",    style->fg[GTK_STATE_NORMAL]));
+	themeColors = g_slist_append (themeColors, render_calculate_theme_color ("GTK-COLOR-BG",    style->bg[GTK_STATE_NORMAL]));
+	themeColors = g_slist_append (themeColors, render_calculate_theme_color ("GTK-COLOR-LIGHT", style->light[GTK_STATE_NORMAL]));
+	themeColors = g_slist_append (themeColors, render_calculate_theme_color ("GTK-COLOR-DARK",  style->dark[GTK_STATE_NORMAL]));
+	themeColors = g_slist_append (themeColors, render_calculate_theme_color ("GTK-COLOR-MID",   style->mid[GTK_STATE_NORMAL]));
+
+	/* Sanity check text+base color as this causes many problems on dark
+	   themes. If brightness distance is not enough we set text to fg/bg
+	   which is always safe. */
+	if (render_get_rgb_distance (&style->base[GTK_STATE_NORMAL], &style->text[GTK_STATE_NORMAL]) > 150) {
+		// FIXME: Use theme labels instead of GTK-COLOR-<something> (e.g. CSS-BACKGROUND)
+		themeColors = g_slist_append (themeColors, render_calculate_theme_color ("GTK-COLOR-BASE", style->base[GTK_STATE_NORMAL]));
+		themeColors = g_slist_append (themeColors, render_calculate_theme_color ("GTK-COLOR-TEXT", style->text[GTK_STATE_NORMAL]));
+	} else {
+		themeColors = g_slist_append (themeColors, render_calculate_theme_color ("GTK-COLOR-BASE", style->bg[GTK_STATE_NORMAL]));
+		themeColors = g_slist_append (themeColors, render_calculate_theme_color ("GTK-COLOR-TEXT", style->fg[GTK_STATE_NORMAL]));
+	}
 
 	color = NULL;
-	gtk_widget_style_get (liferea_shell_get_window (), "link-color", &color, NULL);
+	gtk_widget_style_get (widget, "link-color", &color, NULL);
 	if (color) {
-		themeColors = g_slist_append (themeColors, render_get_theme_color ("GTK-COLOR-NORMAL-LINK", *color));
+		themeColors = g_slist_append (themeColors, render_calculate_theme_color ("GTK-COLOR-NORMAL-LINK", *color));
 		debug0 (DEBUG_HTML, "successfully set the color for links");
 		gdk_color_free (color);
 	}
 
 	color = NULL;
-	gtk_widget_style_get (liferea_shell_get_window (), "visited-link-color", &color, NULL);
+	gtk_widget_style_get (widget, "visited-link-color", &color, NULL);
 	if (color) {
-		themeColors = g_slist_append (themeColors, render_get_theme_color("GTK-COLOR-VISITED-LINK", *color));
+		themeColors = g_slist_append (themeColors, render_calculate_theme_color("GTK-COLOR-VISITED-LINK", *color));
 		debug0 (DEBUG_HTML, "successfully set the color for visited links");
 		gdk_color_free (color);
 	}
+
+	/* As there doesn't seem to be a safe way to determine wether we have a 
+	   dark GTK theme, let's guess it from the foreground vs. background
+	   color average */
+
+	textAvg = style->text[GTK_STATE_NORMAL].red / 256 +
+	        style->text[GTK_STATE_NORMAL].green / 256 +
+	        style->text[GTK_STATE_NORMAL].blue / 256;
+
+	bgAvg = style->bg[GTK_STATE_NORMAL].red / 256 +
+	        style->bg[GTK_STATE_NORMAL].green / 256 +
+	        style->bg[GTK_STATE_NORMAL].blue / 256;
+
+	if (textAvg > bgAvg) {
+		debug0 (DEBUG_HTML, "Dark GTK theme detected.");
+		darkTheme = TRUE;
+	} 
+
+	if (darkTheme) {
+		themeColors = g_slist_append (themeColors, render_calculate_theme_color ("FEEDLIST_UNREAD_BG", style->text[GTK_STATE_NORMAL]));
+		/* Try nice foreground with 'fg' color (note: distance 50 is enough because it should be non-intrusive) */
+		if (render_get_rgb_distance (&style->text[GTK_STATE_NORMAL], &style->fg[GTK_STATE_NORMAL]) > 50)
+			themeColors = g_slist_append (themeColors, render_calculate_theme_color ("FEEDLIST_UNREAD_FG", style->fg[GTK_STATE_NORMAL]));
+		else
+			themeColors = g_slist_append (themeColors, render_calculate_theme_color ("FEEDLIST_UNREAD_FG", style->bg[GTK_STATE_NORMAL]));
+	} else {
+		themeColors = g_slist_append (themeColors, render_calculate_theme_color ("FEEDLIST_UNREAD_FG", style->bg[GTK_STATE_NORMAL]));
+		/* Try nice foreground with 'dark' color (note: distance 50 is enough because it should be non-intrusive) */
+		if (render_get_rgb_distance (&style->dark[GTK_STATE_NORMAL], &style->bg[GTK_STATE_NORMAL]) > 50)
+			themeColors = g_slist_append (themeColors, render_calculate_theme_color ("FEEDLIST_UNREAD_BG", style->dark[GTK_STATE_NORMAL]));
+		else
+			themeColors = g_slist_append (themeColors, render_calculate_theme_color ("FEEDLIST_UNREAD_BG", style->fg[GTK_STATE_NORMAL]));
+	}
 }
 
 static gchar *
@@ -225,6 +287,34 @@ render_set_theme_colors (gchar *css)
 	return css;
 }
 
+const gchar *
+render_get_theme_color (const gchar *name)
+{
+	GSList	*iter;
+	
+	if (!themeColors)
+		return NULL;
+
+	iter = themeColors;
+	while (iter) {
+		themeColorPtr tc = (themeColorPtr)iter->data;
+		if (g_str_equal (name, tc->name))
+			return tc->value;
+		iter = g_slist_next (iter);
+	}
+
+	return NULL;
+}
+
+gboolean
+render_is_dark_theme (void)
+{
+	if (!themeColors)
+		return FALSE;
+
+	return darkTheme;
+}
+
 const gchar *
 render_get_css (gboolean externalCss)
 {
@@ -235,7 +325,7 @@ render_get_css (gboolean externalCss)
 		gchar	*tmp;
 
 		if (!themeColors)
-			render_get_theme_colors();
+			return NULL;
 
 		css = g_string_new(NULL);
 
@@ -251,7 +341,7 @@ render_get_css (gboolean externalCss)
 
 		g_free(defaultStyleSheetFile);
 
-		userStyleSheetFile = g_build_filename (common_get_cache_path (), "liferea.css", NULL);
+		userStyleSheetFile = common_create_config_filename ("liferea.css");
 
 		if (g_file_get_contents(userStyleSheetFile, &tmp, NULL, NULL)) {
 			tmp = render_set_theme_colors(tmp);
@@ -271,8 +361,8 @@ render_get_css (gboolean externalCss)
 		g_free(adblockStyleSheetFile);
 
 		if (externalCss) {
-			/* dump CSS to cache file and create a <style> tag to use the it */
-			gchar *filename = common_create_cache_filename("cache", "style", "css");
+			/* dump CSS to cache file and create a <style> tag to use it */
+			gchar *filename = common_create_cache_filename (NULL, "style", "css");
 			if (!g_file_set_contents(filename, css->str, -1, NULL))
 				g_warning("Cannot write temporary CSS file \"%s\"!", filename);
 
@@ -384,10 +474,3 @@ render_parameter_add (renderParamPtr paramSet, const gchar *fmt, ...)
 
 	g_free (new);
 }
-
-void
-render_parameter_free (renderParamPtr paramSet)
-{
-	g_strfreev (paramSet->params);
-	g_free (paramSet);
-}
