@@ -1,7 +1,7 @@
 /**
  * @file rule.c  item matching rules used by search folders
  *
- * Copyright (C) 2003-2010 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2012 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,11 +24,13 @@
 
 #include "common.h"
 #include "debug.h"
+#include "metadata.h"
 
 #define ITEM_MATCH_RULE_ID		"exact"
 #define ITEM_TITLE_MATCH_RULE_ID	"exact_title"
 #define ITEM_DESC_MATCH_RULE_ID		"exact_desc"
- 
+#define FEED_TITLE_MATCH_RULE_ID	"feed_title"
+
 /** list of available search folder rules */
 static GSList *ruleFunctions = NULL;
 
@@ -80,13 +82,13 @@ rule_free (rulePtr rule)
 static gboolean
 rule_check_item_title (rulePtr rule, itemPtr item)
 {
-	return (NULL != g_strstr_len (item->title, -1, rule->value));
+	return (NULL != item->title && NULL != g_strstr_len (item->title, -1, rule->value));
 }
 
 static gboolean
 rule_check_item_description (rulePtr rule, itemPtr item)
 {
-	return (NULL != g_strstr_len (item->description, -1, rule->value));
+	return (NULL != item->description && NULL != g_strstr_len (item->description, -1, rule->value));
 }
 
 static gboolean
@@ -110,13 +112,33 @@ rule_check_item_is_flagged (rulePtr rule, itemPtr item)
 static gboolean
 rule_check_item_has_enc (rulePtr rule, itemPtr item)
 {
-	return FALSE; // FIXME
+	return item->hasEnclosure;
 }
 
 static gboolean
 rule_check_item_category (rulePtr rule, itemPtr item)
 {
-	return FALSE; // FIXME
+	GSList	*iter = metadata_list_get_values (item->metadata, "category");
+
+	while (iter) {
+		if (g_str_equal (rule->value, (gchar *)iter->data))
+			return TRUE;
+
+		iter = g_slist_next (iter);
+	}
+
+	return FALSE;
+}
+
+static gboolean
+rule_check_feed_title (rulePtr rule, itemPtr item)
+{
+	nodePtr feedNode = node_from_id (item->parentNodeId);
+
+	if (!feedNode)
+		return FALSE;
+
+	return (NULL != feedNode->title && NULL != g_strstr_len (feedNode->title, -1, rule->value));
 }
 
 /* rule initialization */
@@ -156,6 +178,7 @@ rule_init (void)
 	rule_info_add (rule_check_item_is_flagged,	"flagged",			_("Flag status"),	_("is flagged"),	_("is unflagged"),	FALSE);
 	rule_info_add (rule_check_item_has_enc,		"enclosure",			_("Podcast"),		_("included"),		_("not included"),	FALSE);
 	rule_info_add (rule_check_item_category,	"category",			_("Category"),		_("is set"),		_("is not set"),	TRUE);
+	rule_info_add (rule_check_feed_title,		FEED_TITLE_MATCH_RULE_ID,	_("Feed title"),	_("does contain"),	_("does not contain"),	TRUE);
 
 	debug_exit ("rule_init");
 }
