@@ -1,7 +1,7 @@
 /**
- * @file subscription.c common subscription handling
+ * @file subscription.c  common subscription handling
  * 
- * Copyright (C) 2003-2008 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2012 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 
 #include <string.h>
 
+#include "auth.h"
 #include "common.h"
 #include "conf.h"
 #include "db.h"
@@ -33,7 +34,7 @@
 #include "ui/auth_dialog.h"
 #include "ui/itemview.h"
 #include "ui/liferea_shell.h"
-#include "ui/ui_node.h"
+#include "ui/feed_list_node.h"
 
 /* The allowed feed protocol prefixes (see http://25hoursaday.com/draft-obasanjo-feed-URI-scheme-02.html) */
 #define FEED_PROTOCOL_PREFIX "feed://"
@@ -96,8 +97,8 @@ subscription_new (const gchar *source,
 	return subscription;
 }
 
-/* Checks wether updating a feed makes sense. */
-gboolean
+/* Checks whether updating a feed makes sense. */
+static gboolean
 subscription_can_be_updated (subscriptionPtr subscription)
 {
 	if (subscription->updateJob) {
@@ -124,7 +125,6 @@ subscription_reset_update_counter (subscriptionPtr subscription, GTimeVal *now)
 		return;
 		
 	subscription->updateState->lastPoll.tv_sec = now->tv_sec;
-	db_update_state_save (subscription->node->id, subscription->updateState);
 	debug1 (DEBUG_UPDATE, "Resetting last poll counter to %ld.", subscription->updateState->lastPoll.tv_sec);
 }
 
@@ -134,7 +134,7 @@ subscription_favicon_downloaded (gpointer user_data)
 	nodePtr	node = (nodePtr)user_data;
 
 	node_set_icon (node, favicon_load_from_cache (node->id));
-	ui_node_update (node->id);
+	feed_list_node_update (node->id);
 }
 
 void
@@ -143,8 +143,7 @@ subscription_update_favicon (subscriptionPtr subscription)
 	debug1 (DEBUG_UPDATE, "trying to download favicon.ico for \"%s\"", node_get_title (subscription->node));
 	liferea_shell_set_status_bar (_("Updating favicon for \"%s\""), node_get_title (subscription->node));
 	g_get_current_time (&subscription->updateState->lastFaviconPoll);
-	db_update_state_save (subscription->node->id, subscription->updateState);
-	favicon_download (subscription->node->id,
+	favicon_download (subscription,
 	                  node_get_base_url (subscription->node),
 			  subscription_get_source (subscription),
 			  subscription->updateOptions,		// FIXME: correct?
@@ -204,14 +203,16 @@ subscription_process_update_result (const struct updateResult * const result, gp
 
 	/* 1. preprocessing */
 
+	g_assert (subscription->updateJob);
 	/* update the subscription URL on permanent redirects */
-	if ((301 == result->httpstatus) && result->source && g_str_equal (result->source, subscription_get_source (subscription))) {
+	if ((301 == result->httpstatus) && result->source && !g_str_equal (result->source, subscription->updateJob->request->source)) {
+		debug2 (DEBUG_UPDATE, "The URL of \"%s\" has changed permanently and was updated with \"%s\"", node_get_title(node), result->source);
 		subscription_set_source (subscription, result->source);
 		liferea_shell_set_status_bar (_("The URL of \"%s\" has changed permanently and was updated"), node_get_title(node));
 	}
 
 	if (401 == result->httpstatus) { /* unauthorized */
-		ui_auth_dialog_new (subscription, flags);		
+		auth_dialog_new (subscription, flags);
 	} else if (410 == result->httpstatus) { /* gone */
 		subscription->discontinued = TRUE;
 		node->available = TRUE;
@@ -225,6 +226,8 @@ subscription_process_update_result (const struct updateResult * const result, gp
 
 	subscription_update_error_status (subscription, result->httpstatus, result->returncode, result->filterErrors);
 
+	subscription->updateJob = NULL;
+
 	/* 2. call subscription type specific processing */
 	if (processing)
 		SUBSCRIPTION_TYPE (subscription)->process_update_result (subscription, result, flags);
@@ -236,16 +239,19 @@ subscription_process_update_result (const struct updateResult * const result, gp
 		subscription_update_favicon (subscription);
 	
 	/* 4. generic postprocessing */
-	subscription->updateJob = NULL;
 
 	update_state_set_lastmodified (subscription->updateState, update_state_get_lastmodified (result->updateState));
 	update_state_set_cookies (subscription->updateState, update_state_get_cookies (result->updateState));
+	update_state_set_etag (subscription->updateState, update_state_get_etag (result->updateState));
 	g_get_current_time (&subscription->updateState->lastPoll);
-	db_update_state_save (subscription->node->id, subscription->updateState);
 	
 	itemview_update_node_info (subscription->node);
 	itemview_update ();
-	ui_node_update (subscription->node->id);
+
+	feed_list_node_update (subscription->node->id);
+
+	db_subscription_update (subscription);
+	db_node_update (subscription->node);
 	feedlist_schedule_save ();
 }
 
@@ -273,8 +279,6 @@ subscription_update (subscriptionPtr subscription, guint flags)
 		request->updateState = update_state_copy (subscription->updateState);
 		request->options = update_options_copy (subscription->updateOptions);
 		request->source = g_strdup (subscription_get_source (subscription));
-		request->allowRetries = (flags & FEED_REQ_ALLOW_RETRIES)? 1 : 0;
-
 		if (subscription_get_filter (subscription))
 			request->filtercmd = g_strdup (subscription_get_filter (subscription));
 
@@ -297,14 +301,11 @@ subscription_auto_update (subscriptionPtr subscription)
 
 	interval = subscription_get_update_interval (subscription);
 	if (-1 == interval)
-		interval = conf_get_int_value (DEFAULT_UPDATE_INTERVAL);
+		conf_get_int_value (DEFAULT_UPDATE_INTERVAL, &interval);
 			
 	if (-2 >= interval || 0 == interval)
 		return;		/* don't update this subscription */
 		
-	if (conf_get_bool_value (ENABLE_FETCH_RETRIES))
-		flags |= FEED_REQ_ALLOW_RETRIES;
-
 	g_get_current_time (&now);
 	
 	if (subscription->updateState->lastPoll.tv_sec + interval*60 <= now.tv_sec)
@@ -354,7 +355,7 @@ subscription_set_default_update_interval (subscriptionPtr subscription, guint in
 	subscription->defaultInterval = interval;
 }
 
-const gchar *
+static const gchar *
 subscription_get_orig_source (subscriptionPtr subscription)
 {
 	return subscription->origSource; 
@@ -378,7 +379,7 @@ subscription_get_filter (subscriptionPtr subscription)
 	return subscription->filtercmd;
 }
 
-void
+static void
 subscription_set_orig_source (subscriptionPtr subscription, const gchar *source)
 {
 	g_free (subscription->origSource);
@@ -434,6 +435,22 @@ subscription_set_filter (subscriptionPtr subscription, const gchar *filter)
 	feedlist_schedule_save ();
 }
 
+void
+subscription_set_auth_info (subscriptionPtr subscription,
+                            const gchar *username,
+                            const gchar *password)
+{
+	g_assert (NULL != subscription->updateOptions);
+		
+	g_free (subscription->updateOptions->username);
+	g_free (subscription->updateOptions->password);
+
+	subscription->updateOptions->username = g_strdup (username);
+	subscription->updateOptions->password = g_strdup (password);
+
+	liferea_auth_info_store (subscription);
+}
+
 subscriptionPtr
 subscription_import (xmlNodePtr xml, gboolean trusted)
 {
@@ -514,11 +531,13 @@ subscription_export (subscriptionPtr subscription, xmlNodePtr xml, gboolean trus
 
 		if (subscription->updateOptions->dontUseProxy)
 			xmlNewProp (xml, BAD_CAST"dontUseProxy", BAD_CAST"true");
-			
-		if (subscription->updateOptions->username)
-			xmlNewProp (xml, BAD_CAST"username", subscription->updateOptions->username);
-		if (subscription->updateOptions->password)
-			xmlNewProp (xml, BAD_CAST"password", subscription->updateOptions->password);
+		
+		if (!liferea_auth_has_active_store ()) {
+			if (subscription->updateOptions->username)
+				xmlNewProp (xml, BAD_CAST"username", subscription->updateOptions->username);
+			if (subscription->updateOptions->password)
+				xmlNewProp (xml, BAD_CAST"password", subscription->updateOptions->password);
+		}
 	}
 	
 	g_free (interval);
