@@ -1,7 +1,7 @@
 /**
  * @file enclosure-list-view.c enclosures/podcast handling GUI
  *
- * Copyright (C) 2005-2011 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2005-2013 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,8 +29,9 @@
 #include "item.h"
 #include "metadata.h"
 #include "ui/liferea_dialog.h"
+#include "ui/media_player.h"
 #include "ui/popup_menu.h"
-#include "ui/ui_prefs.h"
+#include "ui/ui_common.h"
 
 /* enclosure list view implementation */
 
@@ -40,7 +41,7 @@ enum {
 	ES_DOWNLOADED,
 	ES_SIZE,
 	ES_SIZE_STR,
-	ES_PTR,
+	ES_SERIALIZED,
 	ES_LEN
 };
 
@@ -50,6 +51,7 @@ struct EnclosureListViewPrivate {
 	GSList		*enclosures;		/**< list of currently presented enclosures */
 
 	GtkWidget	*container;		/**< container the list is embedded in */
+	GtkWidget	*expander;		/**< expander that shows/hides the list */
 	GtkWidget	*treeview;
 	GtkTreeStore	*treestore;
 };
@@ -88,6 +90,19 @@ enclosure_list_view_init (EnclosureListView *elv)
 	elv->priv = ENCLOSURE_LIST_VIEW_GET_PRIVATE (elv);
 }
 
+static enclosurePtr
+enclosure_list_view_get_selected_enclosure (EnclosureListView *elv, GtkTreeIter *iter)
+{
+	gchar		*str;
+	enclosurePtr	enclosure;	
+
+	gtk_tree_model_get (GTK_TREE_MODEL (elv->priv->treestore), iter, ES_SERIALIZED, &str, -1);
+	enclosure = enclosure_from_string (str);
+	g_free (str);
+
+	return enclosure;
+}
+
 static gboolean
 on_enclosure_list_button_press (GtkWidget *treeview, GdkEventButton *event, gpointer user_data)
 {
@@ -106,12 +121,8 @@ on_enclosure_list_button_press (GtkWidget *treeview, GdkEventButton *event, gpoi
 	if (!gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview), (gint)event->x, (gint)event->y, &path, NULL, NULL, NULL))
 		return FALSE;
 
-	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (elv->priv->treestore), &iter, path)) {
-		enclosurePtr enclosure;
-		
-		gtk_tree_model_get (GTK_TREE_MODEL (elv->priv->treestore), &iter, ES_PTR, &enclosure, -1);
-		ui_popup_enclosure_menu (enclosure, eb->button, eb->time);
-	}
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (elv->priv->treestore), &iter, path))
+		ui_popup_enclosure_menu (enclosure_list_view_get_selected_enclosure (elv, &iter), eb->button, eb->time);
 	
 	return TRUE;
 }
@@ -122,11 +133,10 @@ on_enclosure_list_popup_menu (GtkWidget *widget, gpointer user_data)
 	GtkTreeView		*treeview = GTK_TREE_VIEW (widget);
 	GtkTreeModel		*model;
 	GtkTreeIter		iter;
+	EnclosureListView 	*elv = (EnclosureListView *)user_data;
 
 	if (gtk_tree_selection_get_selected (gtk_tree_view_get_selection (treeview), &model, &iter)) {
-		enclosurePtr enclosure;	
-		gtk_tree_model_get (model, &iter, ES_PTR, &enclosure, -1);
-		ui_popup_enclosure_menu (enclosure, 3, 0);
+		ui_popup_enclosure_menu (enclosure_list_view_get_selected_enclosure (elv, &iter), 3, 0);
 		return TRUE;
 	}
 	
@@ -136,15 +146,16 @@ on_enclosure_list_popup_menu (GtkWidget *widget, gpointer user_data)
 static gboolean
 on_enclosure_list_activate (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, gpointer user_data)
 {
-	enclosurePtr	enclosure;
-	GtkTreeIter	iter;
-	GtkTreeModel	*model;
+	GtkTreeIter		iter;
+	GtkTreeModel		*model;
+	EnclosureListView 	*elv = (EnclosureListView *)user_data;
 		
-	gtk_tree_selection_get_selected (gtk_tree_view_get_selection (treeview), &model, &iter);
-	gtk_tree_model_get (model, &iter, ES_PTR, &enclosure, -1);
-	on_popup_open_enclosure (enclosure);
+	if (gtk_tree_selection_get_selected (gtk_tree_view_get_selection (treeview), &model, &iter)) {
+		on_popup_open_enclosure (enclosure_list_view_get_selected_enclosure (elv, &iter));
+		return TRUE;
+	}
 
-	return TRUE;
+	return FALSE;
 }
 
 EnclosureListView *
@@ -156,7 +167,13 @@ enclosure_list_view_new ()
 	GtkWidget		*widget;
 		
 	elv = ENCLOSURE_LIST_VIEW (g_object_new (ENCLOSURE_LIST_VIEW_TYPE, NULL));
-	elv->priv->container = gtk_expander_new (_("Attachments"));	
+
+	/* Use a vbox to allow media player insertion */
+	elv->priv->container = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+	gtk_widget_set_name (GTK_WIDGET (elv->priv->container), "enclosureview");
+
+	elv->priv->expander = gtk_expander_new (_("Attachments"));
+	gtk_box_pack_end (GTK_BOX (elv->priv->container), elv->priv->expander, TRUE, TRUE, 0);
 	
 	widget = gtk_scrolled_window_new (NULL, NULL);
 	/* FIXME: Setting a fixed size is not nice, but a workaround for the
@@ -164,7 +181,7 @@ enclosure_list_view_new ()
 	gtk_widget_set_size_request (widget, -1, 75);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (widget), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (widget), GTK_SHADOW_IN);
-	gtk_container_add (GTK_CONTAINER (elv->priv->container), widget);
+	gtk_container_add (GTK_CONTAINER (elv->priv->expander), widget);
 
 	elv->priv->treeview = gtk_tree_view_new ();
 	gtk_container_add (GTK_CONTAINER (widget), elv->priv->treeview);
@@ -177,7 +194,7 @@ enclosure_list_view_new ()
 						   G_TYPE_BOOLEAN,	/* ES_DOWNLOADED */
 						   G_TYPE_ULONG,	/* ES_SIZE */
 						   G_TYPE_STRING,	/* ES_SIZE_STRING */
-						   G_TYPE_POINTER	/* ES_PTR */
+						   G_TYPE_STRING	/* ES_SERIALIZED */
 	                                           );
 	gtk_tree_view_set_model (GTK_TREE_VIEW (elv->priv->treeview), GTK_TREE_MODEL(elv->priv->treestore));
 
@@ -224,9 +241,14 @@ enclosure_list_view_get_widget (EnclosureListView *elv)
 void
 enclosure_list_view_load (EnclosureListView *elv, itemPtr item)
 {
-	GSList		*list;
+	GSList		*list, *filteredList;
 	guint		len;
 
+	/* FIXME: this is ugly, needed due to race between EnclosureListView
+	   setup by LifereaShell and last item selection */
+	if (!elv)
+		return;
+
 	/* cleanup old content */
 	gtk_tree_store_clear (elv->priv->treestore);
 	list = elv->priv->enclosures;
@@ -237,9 +259,62 @@ enclosure_list_view_load (EnclosureListView *elv, itemPtr item)
 	g_slist_free (elv->priv->enclosures);
 	elv->priv->enclosures = NULL;	
 	
-	/* decide visibility of the list */
+	/* load list into tree view */
+	filteredList = NULL;
 	list = metadata_list_get_values (item->metadata, "enclosure");
-	len = g_slist_length (list);
+	while (list) {
+		enclosurePtr enclosure = enclosure_from_string (list->data);
+		if (enclosure) {
+			// Filter unwanted MIME types (we only want audio/* and video/*)
+			if (enclosure->mime &&
+                            (g_str_has_prefix (enclosure->mime, "video/") ||
+			    (g_str_has_prefix (enclosure->mime, "audio/")))) {
+				GtkTreeIter	iter;
+				gchar		*sizeStr;
+				guint		size = enclosure->size;
+
+				/* The following literals are the enclosure list size units */
+				gchar *unit = _(" Bytes");
+				if (size > 1024) {
+					size /= 1024;
+					unit = _("kB");
+				}
+				if (size > 1024) {
+					size /= 1024;
+					unit = _("MB");
+				}
+				if (size > 1024) {
+					size /= 1024;
+					unit = _("GB");
+				}			
+			
+				/* The following literal is the format string for enclosure sizes (number + unit string) */
+				if (size > 0)
+					sizeStr = g_strdup_printf (_("%d%s"), size, unit);
+				else
+					sizeStr = g_strdup ("");
+
+				gtk_tree_store_append (elv->priv->treestore, &iter, NULL);
+				gtk_tree_store_set (elv->priv->treestore, &iter, 
+					            ES_NAME_STR, enclosure->url,
+						    ES_MIME_STR, enclosure->mime?enclosure->mime:"",
+					            ES_DOWNLOADED, enclosure->downloaded,
+						    ES_SIZE, enclosure->size,
+						    ES_SIZE_STR, sizeStr,
+						    ES_SERIALIZED, list->data,
+						    -1);
+				g_free (sizeStr);
+
+				elv->priv->enclosures = g_slist_append (elv->priv->enclosures, enclosure);
+				filteredList = g_slist_append (filteredList, list->data);
+			}
+		}
+		
+		list = g_slist_next (list);
+	}
+
+	/* decide visibility of the list */
+	len = g_slist_length (elv->priv->enclosures);
 	if (len == 0) {
 		enclosure_list_view_hide (elv);
 		return;
@@ -249,56 +324,22 @@ enclosure_list_view_load (EnclosureListView *elv, itemPtr item)
 
 	/* update list title */
 	gchar *text = g_strdup_printf (ngettext("%d attachment", "%d attachments", len), len);
-	gtk_expander_set_label (GTK_EXPANDER (elv->priv->container), text);
+	gtk_expander_set_label (GTK_EXPANDER (elv->priv->expander), text);
 	g_free (text);
 
-	/* load list into tree view */	
-	while (list) {
-		gchar *sizeStr;
-		enclosurePtr enclosure;
-		GtkTreeIter iter;
-		
-		enclosure = enclosure_from_string (list->data);
-		if (enclosure) {
-			guint size = enclosure->size;
-			
-			/* The following literals are the enclosure list size units */
-			gchar *unit = _(" Bytes");
-			if (size > 1024) {
-				size /= 1024;
-				unit = _("kB");
-			}
-			if (size > 1024) {
-				size /= 1024;
-				unit = _("MB");
-			}
-			if (size > 1024) {
-				size /= 1024;
-				unit = _("GB");
-			}			
-			
-			/* The following literal is the format string for enclosure sizes (number + unit string) */
-			if (size > 0)
-				sizeStr = g_strdup_printf (_("%d%s"), size, unit);
-			else
-				sizeStr = g_strdup ("");
-			
-			gtk_tree_store_append (elv->priv->treestore, &iter, NULL);
-			gtk_tree_store_set (elv->priv->treestore, &iter, 
-			                    ES_NAME_STR, enclosure->url,
-					    ES_MIME_STR, enclosure->mime?enclosure->mime:"",
-			                    ES_DOWNLOADED, enclosure->downloaded,
-					    ES_SIZE, enclosure->size,
-					    ES_SIZE_STR, sizeStr,
-					    ES_PTR, enclosure,
-					    -1);
-			g_free (sizeStr);
+	/* Load the optional media player plugin */
+	liferea_media_player_load (elv->priv->container, filteredList);
+}
 
-			elv->priv->enclosures = g_slist_append (elv->priv->enclosures, enclosure);
-		}
-		
-		list = list->next;
-	}
+void
+enclosure_list_view_select (EnclosureListView *elv, guint position)
+{
+	GtkTreeIter iter;
+
+	if (!gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (elv->priv->treestore), &iter, NULL, position))
+		return;
+
+	gtk_tree_selection_select_iter (gtk_tree_view_get_selection (GTK_TREE_VIEW (elv->priv->treeview)), &iter);
 }
 
 void
@@ -369,7 +410,6 @@ on_adddialog_response (GtkDialog *dialog, gint response_id, gpointer user_data)
 		}
 		etp->cmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (liferea_dialog_lookup (GTK_WIDGET (dialog), "enc_cmd_entry"))));
 		etp->permanent = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (liferea_dialog_lookup (GTK_WIDGET (dialog), "enc_always_btn")));
-		etp->remote = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (liferea_dialog_lookup (GTK_WIDGET (dialog), "enc_remote_open_btn")));
 		if (new)
 			enclosure_mime_type_add (etp);
 		else
@@ -397,7 +437,6 @@ ui_enclosure_type_setup (encTypePtr type, enclosurePtr enclosure, gchar *typestr
 		typestr = type->mime?type->mime:type->extension;
 		gtk_entry_set_text (GTK_ENTRY (liferea_dialog_lookup (dialog, "enc_cmd_entry")), type->cmd);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (liferea_dialog_lookup (dialog, "enc_always_btn")), TRUE);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (liferea_dialog_lookup (GTK_WIDGET(dialog), "enc_remote_open_btn")), type->remote);
 	}
 
 	if (!strchr(typestr, '/')) 
@@ -419,7 +458,7 @@ ui_enclosure_type_setup (encTypePtr type, enclosurePtr enclosure, gchar *typestr
 void
 on_popup_open_enclosure (gpointer callback_data)
 {
-	gchar		*typestr;
+	gchar		*typestr, *tmp = NULL;
 	enclosurePtr	enclosure = (enclosurePtr)callback_data;
 	gboolean	found = FALSE;
 	encTypePtr	etp = NULL;
@@ -430,7 +469,7 @@ on_popup_open_enclosure (gpointer callback_data)
 	/* find extension by looking for last '.' */
 	typestr = strrchr (enclosure->url, '.');
 	if (typestr)
-		typestr = g_strdup (typestr + 1);
+		typestr = tmp = g_strdup (typestr + 1);
 
 	/* handle case where there is a slash after the '.' */
 	if (typestr && strrchr (typestr, '/'))
@@ -442,7 +481,7 @@ on_popup_open_enclosure (gpointer callback_data)
 	
 	/* if we found no extension we map to dummy type "data" */
 	if (!typestr)
-		typestr = g_strdup ("data");
+		typestr = tmp = g_strdup ("data");
 		
 	/* strip GET parameters from typestr */	
 	g_strdelimit (typestr, "?", 0);
@@ -474,7 +513,7 @@ on_popup_open_enclosure (gpointer callback_data)
 	}
 	
 	if (found) {
-		enclosure_save_as_file (etp, enclosure->url, NULL);
+		enclosure_download (etp, enclosure->url, TRUE);
 	} else {
 		if (enclosure->mime)
 			ui_enclosure_type_setup (NULL, enclosure, enclosure->mime);
@@ -482,41 +521,15 @@ on_popup_open_enclosure (gpointer callback_data)
 			ui_enclosure_type_setup (NULL, enclosure, typestr);
 	}
 		
-	g_free (typestr);
-}
-
-static void
-on_encsave_clicked (const gchar *filename, gpointer user_data)
-{
-	gchar		*url = (gchar *)user_data;
-	gchar		*utfname;
-	
-	if (!filename)
-		return;
-
-	utfname = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
-	enclosure_save_as_file (NULL, url, utfname);
-	g_free (utfname);
+	g_free (tmp);
 }
 
 void
 on_popup_save_enclosure (gpointer callback_data)
 {
 	enclosurePtr	enclosure = (enclosurePtr)callback_data;
-	gchar		*filename;
 
-	filename = strrchr (enclosure->url, '/');
-	if (filename)
-		filename++; /* Skip the slash to find the filename */
-	else
-		filename = enclosure->url;
-		
-	filename = g_strdup (filename);
-	g_strdelimit (filename, "?", 0);	/* strip GET parameters */
-		
-	ui_choose_file (_("Choose File"), GTK_STOCK_SAVE_AS, TRUE, on_encsave_clicked, NULL, filename, NULL, NULL, enclosure->url);
-
-	g_free (filename);
+	enclosure_download (NULL, enclosure->url, TRUE);
 }
 
 void
