@@ -878,6 +878,7 @@ on_remove_item_activate (GtkMenuItem *menuitem, gpointer user_data)
 	
 	item = itemlist_get_selected ();
 	if (item) {
+		itemview_select_item (NULL);
 		itemlist_remove_item (item);
 	} else {
 		liferea_shell_set_important_status_bar (_("No item has been selected"));
@@ -909,9 +910,11 @@ item_list_view_select (ItemListView *ilv, itemPtr item)
 			itemlist_selection_changed (NULL);
 
 		path = gtk_tree_model_get_path (gtk_tree_view_get_model (treeview), &iter);
-		gtk_tree_view_set_cursor (treeview, path, NULL, FALSE);
-		gtk_tree_view_scroll_to_cell (treeview, path, NULL, FALSE, 0.0, 0.0);
-		gtk_tree_path_free (path);
+		if (path) {
+			gtk_tree_view_set_cursor (treeview, path, NULL, FALSE);
+			gtk_tree_view_scroll_to_cell (treeview, path, NULL, FALSE, 0.0, 0.0);
+			gtk_tree_path_free (path);
+		}
 	} else {
 		gtk_tree_selection_unselect_all (selection);
 	}
