@@ -1,7 +1,7 @@
 /**
  * @file item_list_view.h  presenting items in a GtkTreeView
  *
- * Copyright (C) 2004-2010 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2004-2012 Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2004-2005 Nathan J. Conrad <t98502@users.sourceforge.net>
  *	      
  * This library is free software; you can redistribute it and/or
@@ -76,7 +76,7 @@ ItemListView * item_list_view_create (GtkWidget *window);
 GtkTreeView * item_list_view_get_widget (ItemListView *ilv);
 
 /**
- * Checks wether the given id is in the ItemListView.
+ * Checks whether the given id is in the ItemListView.
  *
  * @param ilv	the ItemListView
  * @param id	the item id
@@ -94,14 +94,6 @@ gboolean item_list_view_contains_id (ItemListView *ilv, gulong id);
  */
 void item_list_view_set_sort_column (ItemListView *ilv, nodeViewSortType sortType, gboolean sortReversed);
 
-/**
- * Unselect all items in the ItemListView and scroll to top. This 
- * is typically called when changing feed.
- *
- * @param ilv	the ItemListView
- */
-void item_list_view_prefocus (ItemListView *ilv);
-
 /**
  * Selects the given item (if it is in the ItemListView).
  *
@@ -154,27 +146,6 @@ void item_list_view_clear (ItemListView *ilv);
  */
 void item_list_view_update (ItemListView *ilv, gboolean hasEnclosures);
 
-/* item list callbacks */
-
-/**
- * Callback activated when an item is double-clicked. It opens the URL
- * of the item in a web browser.
- */
-void on_Itemlist_row_activated (GtkTreeView     *treeview,
-                                GtkTreePath     *path,
-                                GtkTreeViewColumn *column,
-                                gpointer         user_data);
-
-/**
- * Callback for column selection change.
- */
-void itemlist_sort_column_changed_cb (GtkTreeSortable *treesortable, gpointer user_data);
-
-/**
- * Callback for item list selection change.
- */
-void on_itemlist_selection_changed (GtkTreeSelection *selection, gpointer data);
-
 /* menu callbacks */
 
 /**
@@ -191,12 +162,17 @@ void on_toggle_item_flag (GtkMenuItem *menuitem, gpointer user_data);
 /**
  * Opens the selected item in a browser.
  */
-void on_popup_launchitem_selected (void);
+void on_popup_launch_item_selected (void);
 
 /**
  * Opens the selected item in a browser.
  */
-void on_popup_launchitem_in_tab_selected (void);
+void on_popup_launch_item_in_tab_selected (void);
+
+/**
+ * Opens the selected item in a browser.
+ */
+void on_popup_launch_item_external_selected (void);
 
 /**
  * Toggles the read status of right-clicked item.
@@ -246,10 +222,6 @@ itemPtr item_list_view_find_unread_item (ItemListView *ilv, gulong startId);
  */
 void on_next_unread_item_activate (GtkMenuItem *menuitem, gpointer user_data);
 
-void on_popup_next_unread_item_selected(gpointer callback_data, guint callback_action, GtkWidget *widget);
-
-void on_nextbtn_clicked(GtkButton *button, gpointer user_data);
-
 /**
  * Update a single item of a ItemListView
  *
