@@ -301,6 +301,7 @@ void
 liferea_htmlview_write (LifereaHtmlView *htmlview, const gchar *string, const gchar *base)
 { 
 	const gchar	*baseURL = base;
+	const gchar	*errMsg = "ERROR: Invalid encoded UTF8 buffer passed to HTML widget! This shouldn't happen.";
 
 	if (!htmlview)
 		return;
@@ -318,7 +319,7 @@ liferea_htmlview_write (LifereaHtmlView *htmlview, const gchar *string, const gc
 	
 	if (!g_utf8_validate (string, -1, NULL)) {
 		/* It is really a bug if we get invalid encoded UTF-8 here!!! */
-		g_error ("Invalid encoded UTF8 buffer passed to HTML widget!");
+		(RENDERER (htmlview)->write) (htmlview->priv->renderWidget, errMsg, strlen (errMsg), baseURL, "text/plain");
 	} else {
 		(RENDERER (htmlview)->write) (htmlview->priv->renderWidget, string, strlen (string), baseURL, "application/xhtml+xml");
 	}
