@@ -991,7 +991,7 @@ static const GtkActionEntry liferea_shell_item_action_entries[] = {
 	{"LaunchItemInBrowser", NULL, N_("_Open In Browser"), NULL, N_("Launches the item's link in the Liferea item pane."),
 	 G_CALLBACK(on_popup_launch_item_selected)},
 	{"LaunchItemInExternalBrowser", NULL, N_("Open In _External Browser"), NULL, N_("Launches the item's link in the configured external browser."),
-	 G_CALLBACK(on_popup_launch_item_selected)}
+	 G_CALLBACK(on_popup_launch_item_external_selected)}
 };
 
 static const GtkToggleActionEntry liferea_shell_action_toggle_entries[] = {
@@ -1109,18 +1109,6 @@ liferea_shell_restore_state (const gchar *overrideWindowState)
 	
 	liferea_shell_restore_position ();
 
-	debug0 (DEBUG_GUI, "Loading pane proportions");
-		
-	conf_get_int_value (LAST_VPANE_POS, &last_vpane_pos);
-	if (last_vpane_pos)
-		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("leftpane")), last_vpane_pos);
-	conf_get_int_value (LAST_HPANE_POS, &last_hpane_pos);
-	if (last_hpane_pos)
-		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("normalViewPane")), last_hpane_pos);
-	conf_get_int_value (LAST_WPANE_POS, &last_wpane_pos);
-	if (last_wpane_pos)
-		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("wideViewPane")), last_wpane_pos);
-
 	conf_get_bool_value (SHOW_TRAY_ICON, &show_tray_icon);
 	conf_get_bool_value (START_IN_TRAY, &start_in_tray);
 
@@ -1175,6 +1163,19 @@ liferea_shell_restore_state (const gchar *overrideWindowState)
 			debug0 (DEBUG_GUI, "Restoring window state 'shown'");
 			gtk_widget_show (GTK_WIDGET (shell->priv->window));
 	}
+
+	debug0 (DEBUG_GUI, "Loading pane proportions");
+
+	/* This only works after the window has been restored, so we do it last. */
+	conf_get_int_value (LAST_VPANE_POS, &last_vpane_pos);
+	if (last_vpane_pos)
+		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("leftpane")), last_vpane_pos);
+	conf_get_int_value (LAST_HPANE_POS, &last_hpane_pos);
+	if (last_hpane_pos)
+		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("normalViewPane")), last_hpane_pos);
+	conf_get_int_value (LAST_WPANE_POS, &last_wpane_pos);
+	if (last_wpane_pos)
+		gtk_paned_set_position (GTK_PANED (liferea_shell_lookup ("wideViewPane")), last_wpane_pos);
 }
 
 void
