@@ -1,7 +1,7 @@
 /**
  * @file search-folder-dialog.c  Search folder properties dialog
  *
- * Copyright (C) 2007-2011 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2011 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,9 +25,8 @@
 #include "vfolder.h"
 #include "ui/itemview.h"
 #include "ui/liferea_dialog.h"
-#include "ui/liferea_shell.h"
 #include "ui/rule_editor.h"
-#include "ui/ui_node.h"
+#include "ui/feed_list_node.h"
 
 #define SEARCH_FOLDER_DIALOG_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), SEARCH_FOLDER_DIALOG_TYPE, SearchFolderDialogPrivate))
 
@@ -90,7 +89,7 @@ on_propdialog_response (GtkDialog *dialog, gint response_id, gpointer user_data)
 		if (!sfd->priv->node->parent)
 			feedlist_node_added (sfd->priv->node);
 			
-		ui_node_update (sfd->priv->node->id);
+		feed_list_node_update (sfd->priv->node->id);
 	}
 	
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -117,7 +116,7 @@ search_folder_dialog_new (nodePtr node)
 	sfd->priv->re = rule_editor_new (sfd->priv->vfolder->itemset);
 	
 	/* Create the dialog */
-	dialog = liferea_dialog_new (NULL, "vfolderdialog");
+	dialog = liferea_dialog_new ("search_folder.ui", "vfolderdialog");
 	
 	/* Setup search folder name */
 	sfd->priv->nameEntry = liferea_dialog_lookup (dialog, "searchNameEntry");
