@@ -1,7 +1,7 @@
 /**
  * @file ui_common.c  UI helper functions
  *
- * Copyright (C) 2008-2010 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2008-2012 Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2009 Hubert Figuiere <hub@figuiere.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@
 
 void
 ui_common_setup_combo_menu (GtkWidget *widget,
-                     gchar **options,
+                     const gchar **options,
                      GCallback callback,
                      gint defaultValue)
 {
@@ -158,7 +158,7 @@ ui_choose_file_or_dir(gchar *title, const gchar *buttonName, gboolean saving, gb
 	g_assert (!(defaultFilename && !saving));
 
 	if (!currentPath)
-		conf_get_str_value (ENCLOSURE_DOWNLOAD_PATH, &path);
+		path = g_strdup (g_get_home_dir ());
 	else
 		path = g_strdup (currentPath);
 
@@ -194,9 +194,15 @@ ui_choose_file_or_dir(gchar *title, const gchar *buttonName, gboolean saving, gb
 
 	if (filterstring && filtername) {
 		GtkFileFilter *filter, *allfiles;
+		gchar **filterstrings, **f;
 
 		filter = gtk_file_filter_new ();
-		gtk_file_filter_add_pattern (filter, filterstring);
+
+		filterstrings = g_strsplit (filterstring, "|", 0);
+		for (f = filterstrings; *f != NULL; f++)
+			gtk_file_filter_add_pattern (filter, *f);
+		g_strfreev (filterstrings);
+
 		gtk_file_filter_set_name (filter, filtername);
 		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
 
@@ -215,9 +221,3 @@ ui_choose_file (gchar *title, const gchar *buttonName, gboolean saving, fileChoo
 {
 	ui_choose_file_or_dir (title, buttonName, saving, FALSE, callback, currentPath, defaultFilename, filterstring, filtername, user_data);
 }
-
-void
-ui_choose_directory (gchar *title, const gchar *buttonName, fileChoosenCallback callback, const gchar *currentPath, gpointer user_data)
-{
-	ui_choose_file_or_dir (title, buttonName, FALSE, TRUE, callback, currentPath, NULL, NULL, NULL, user_data);
-}
