@@ -1,7 +1,7 @@
 /**
  * @file ui_dnd.c everything concerning Drag&Drop
  *
- * Copyright (C) 2003-2011 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2012 Lars Windolf <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,13 +20,14 @@
 
 #include <string.h>		/* For strcmp */
 #include "common.h"
+#include "db.h"
 #include "feed.h"
 #include "feedlist.h"
 #include "folder.h"
 #include "debug.h"
 #include "ui/item_list_view.h"
 #include "ui/feed_list_view.h"
-#include "ui/ui_node.h"
+#include "ui/feed_list_node.h"
 #include "ui/ui_dnd.h"
 #include "fl_sources/node_source.h"
 
@@ -34,7 +35,8 @@
     Why does Liferea need such a complex DnD handling (for the feed list)?
 
      -> Because parts of the feed list might be un-draggable.
-     -> Because drag source and target might be different node sources.
+     -> Because drag source and target might be different node sources
+	with even incompatible subscription types.
      -> Because removal at drag source and insertion at drop target
         must be atomic to avoid subscription losses.
 	
@@ -54,7 +56,7 @@ static gboolean (*old_feed_drag_data_received)(GtkTreeDragDest *drag_dest,
 
 /* GtkTreeDragSource/GtkTreeDragDest implementation				*/
 
-/** decides wether a feed cannot be dragged or not */
+/** decides whether a feed cannot be dragged or not */
 static gboolean
 ui_dnd_feed_draggable (GtkTreeDragSource *drag_source, GtkTreePath *path)
 {
@@ -86,7 +88,6 @@ ui_dnd_feed_drop_possible (GtkTreeDragDest *drag_dest, GtkTreePath *dest_path, G
 	nodePtr		sourceNode, targetNode;
 	
 	debug1 (DEBUG_GUI, "DnD check if feed dropping is possible (%d)", dest_path);
-
 		   	
 	if (!(old_feed_drop_possible) (drag_dest, dest_path, selection_data))
 		return FALSE;
@@ -105,7 +106,7 @@ ui_dnd_feed_drop_possible (GtkTreeDragDest *drag_dest, GtkTreePath *dest_path, G
 	   iterator to insert after). In any case we have to check
 	   if it is a writeable node source. */
 
-	/* never drop into read-only subscription node sources */
+	/* Never drop into read-only subscription node sources */
 	if (!(NODE_SOURCE_TYPE (targetNode)->capabilities & NODE_SOURCE_CAPABILITY_WRITABLE_FEEDLIST))
 		return FALSE;
 
@@ -116,6 +117,13 @@ ui_dnd_feed_drop_possible (GtkTreeDragDest *drag_dest, GtkTreePath *dest_path, G
 	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, src_path)) {
 		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, FS_PTR, &sourceNode, -1);
 
+		g_assert (sourceNode);
+
+		/* Never drop into another node source as this arises to many problems
+		   (e.g. remote sync, different subscription type, e.g. SF #2855990) */
+		if (NODE_SOURCE_TYPE (targetNode) != NODE_SOURCE_TYPE (sourceNode))
+			return FALSE;
+
 		if (IS_FOLDER(sourceNode) && !(NODE_SOURCE_TYPE (targetNode)->capabilities & NODE_SOURCE_CAPABILITY_HIERARCHIC_FEEDLIST))
 			return FALSE;
 	}
@@ -146,13 +154,12 @@ ui_dnd_feed_drag_data_received (GtkTreeDragDest *drag_dest, GtkTreePath *dest, G
 			node_update_counters (oldParent);
 			
 			if (0 == g_slist_length (oldParent->children))
-				ui_node_add_empty_node (ui_node_to_iter (oldParent->id));
+				feed_list_node_add_empty_node (feed_list_node_to_iter (oldParent->id));
 			
 			/* and rebuild new parents child list */
 			if (gtk_tree_model_iter_parent (GTK_TREE_MODEL (drag_dest), &parentIter, &iter)) {
 				gtk_tree_model_get (GTK_TREE_MODEL (drag_dest), &parentIter, FS_PTR, &newParent, -1);
 			} else {
-				debug0 (DEBUG_GUI, "Defaulting to feed list root as drop target");
 				gtk_tree_model_get_iter_first (GTK_TREE_MODEL (drag_dest), &parentIter);
 				newParent = feedlist_get_root ();
 			}
@@ -204,12 +211,13 @@ ui_dnd_feed_drag_data_received (GtkTreeDragDest *drag_dest, GtkTreePath *dest, G
 				} else {
 					debug0 (DEBUG_GUI, "   -> removing empty node");
 					/* remove possible existing "(empty)" node from newParent */
-					ui_node_remove_empty_node (&parentIter);
+					feed_list_node_remove_empty_node (&parentIter);
 				}
 				valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (drag_dest), &iter2);
 				pos++;
 			}
 			
+			db_node_update (node);
 			node_update_counters (newParent);
 			
 			feedlist_schedule_save ();
