@@ -1,9 +1,9 @@
 /**
  * @file webkit.c  WebKit browser module for Liferea
  *
- * Copyright (C) 2007-2010 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2010 Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2008 Lars Strojny <lars@strojny.net>
- * Copyright (C) 2009 Emilio Pozuelo Monfort <pochu27@gmail.com>
+ * Copyright (C) 2009-2012 Emilio Pozuelo Monfort <pochu27@gmail.com>
  * Copyright (C) 2009 Adrian Bunk <bunk@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <gconf/gconf-client.h>
 #include <libsoup/soup.h>
 #include <webkit/webkit.h>
 #include <string.h>
@@ -40,23 +39,16 @@ static WebKitWebSettings *settings = NULL;
  * settings object.
  */
 static void
-liferea_webkit_disable_javascript_cb (GConfClient *client,
-				      guint cnxn_id,
-				      GConfEntry *entry,
+liferea_webkit_disable_javascript_cb (GSettings *gsettings,
+				      gchar *key,
 				      gpointer user_data)
 {
-	GConfValue *disable_javascript;
-
-	g_return_if_fail (entry != NULL);
-
-	disable_javascript = gconf_entry_get_value (entry);
-	if (!disable_javascript || disable_javascript->type != GCONF_VALUE_BOOL)
-		return;
+	g_return_if_fail (key != NULL);
 
 	g_object_set (
 		settings,
 		"enable-scripts",
-		!gconf_value_get_bool (disable_javascript),
+		!g_settings_get_boolean (gsettings, key),
 		NULL
 	);
 }
@@ -67,23 +59,16 @@ liferea_webkit_disable_javascript_cb (GConfClient *client,
  * settings object.
  */
 static void
-liferea_webkit_enable_plugins_cb (GConfClient *client,
-				  guint cnxn_id,
-				  GConfEntry *entry,
+liferea_webkit_enable_plugins_cb (GSettings *gsettings,
+				  gchar *key,
 				  gpointer user_data)
 {
-	GConfValue *enable_plugins;
-
-	g_return_if_fail (entry != NULL);
-
-	enable_plugins = gconf_entry_get_value (entry);
-	if (!enable_plugins || enable_plugins->type != GCONF_VALUE_BOOL)
-		return;
+	g_return_if_fail (key != NULL);
 
 	g_object_set (
 		settings,
 		"enable-plugins",
-		gconf_value_get_bool (enable_plugins),
+		g_settings_get_boolean (gsettings, key),
 		NULL
 	);
 }
@@ -99,7 +84,8 @@ webkit_get_font (guint *size)
 	conf_get_str_value (USER_FONT, &font);
 	if (0 == strlen (font)) {
 		g_free (font);
-		conf_get_str_value (DEFAULT_FONT, &font);
+		font = NULL;
+		conf_get_default_font_from_schema (DEFAULT_FONT, &font);
 	}
 
 	if (font) {
@@ -115,13 +101,12 @@ webkit_get_font (guint *size)
 }
 
 /**
- * HTML plugin init method
+ * HTML renderer init method
  */
 static void
 liferea_webkit_init (void)
 {
 	gboolean	disable_javascript, enable_plugins;
-	GConfClient	*client;
 	gchar		*font;
 	guint		fontSize;
 
@@ -166,23 +151,17 @@ liferea_webkit_init (void)
 		NULL
 	);
 
-
-	client = gconf_client_get_default ();
-
-	gconf_client_notify_add (
-		client,
-		DISABLE_JAVASCRIPT,
-		liferea_webkit_disable_javascript_cb,
-		NULL, NULL, NULL
-	);
-	gconf_client_notify_add (
-		client,
-		ENABLE_PLUGINS,
-		liferea_webkit_enable_plugins_cb,
-		NULL, NULL, NULL
+	conf_signal_connect (
+		"changed::" DISABLE_JAVASCRIPT,
+		G_CALLBACK (liferea_webkit_disable_javascript_cb),
+		NULL
 	);
 
-	g_object_unref (client);
+	conf_signal_connect (
+		"changed::" ENABLE_PLUGINS,
+		G_CALLBACK (liferea_webkit_enable_plugins_cb),
+		NULL
+	);
 }
 
 /**
@@ -346,12 +325,19 @@ static WebKitWebView*
 webkit_create_web_view (WebKitWebView *view, WebKitWebFrame *frame)
 {
 	LifereaHtmlView *htmlview;
-	GtkWidget	*scrollpane;
+	GtkWidget	*container;
 	GtkWidget	*htmlwidget;
+	GList 		*children;
 
 	htmlview = browser_tabs_add_new (NULL, NULL, TRUE);
-	scrollpane = liferea_htmlview_get_widget (htmlview);
-	htmlwidget = gtk_bin_get_child (GTK_BIN (scrollpane));
+	container = liferea_htmlview_get_widget (htmlview);
+
+	/* Ugly lookup of the webview. LifereaHtmlView uses a GtkBox
+	   with first a URL bar (sometimes invisble) and the HTML renderer
+	   as 2nd child */
+	children = gtk_container_get_children (GTK_CONTAINER (container));
+	htmlwidget = gtk_bin_get_child (GTK_BIN (children->next->data));
+
 	return WEBKIT_WEB_VIEW (htmlwidget);
 }
 
