@@ -1,7 +1,7 @@
 /**
  * @file xml.c XML helper methods for Liferea
  * 
- * Copyright (C) 2003-2009  Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2013  Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006  Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -167,44 +167,30 @@ xhtml_extract (xmlNodePtr xml, gint xhtmlMode, const gchar *defaultBase)
 	return result;
 }
 
-/* Convert the given string to proper XHTML content.*/
+/*
+ * Read HTML string and convert to XHTML, placing in a div tag 
+ */
 gchar *
-xhtml_from_text (const gchar *sourceText)
+xhtml_extract_from_string (const gchar *html, const gchar *defaultBase)
 {
-	gchar		*text, *result = NULL;
-	xmlDocPtr	doc = NULL;
-	xmlBufferPtr	buf;
-	xmlNodePtr 	body, cur;
+	xmlDocPtr doc = NULL;
+	xmlNodePtr body = NULL;
+	gchar *result;
 	
-	if (!sourceText)
-		return g_strdup ("");
-	
-	text = g_strstrip (g_strdup (sourceText));	/* stripping whitespaces to make empty string detection easier */
-	if (*text) {	
-		doc = xhtml_parse (text, strlen (text));
-		
-		buf = xmlBufferCreate ();
+	/* never process empty content, xmlDocCopy() doesn't like it... */
+	if (html != NULL && !common_str_is_empty (html)) {
+		doc = xhtml_parse (html, strlen (html));
 		body = xhtml_find_body (doc);
-		if(body) {
-			cur = body->xmlChildrenNode;
-			while(cur) {
-				xmlNodeDump(buf, doc, cur, 0, 0 );
-				cur = cur->next;
-			}
-		}
-
-		if(xmlBufferLength(buf) > 0)
-			result = g_strdup(xmlBufferContent(buf));
-		else
-			result = g_strdup("");
-
-		xmlBufferFree(buf);
-		xmlFreeDoc(doc);
-		g_free(text);
-	} else {
-		result = text;
+		if (body == NULL) 
+			body = xmlDocGetRootElement (doc);
 	}
 	
+	if (body != NULL)
+		result = xhtml_extract (body, 1, defaultBase);
+	else
+		result = xmlCharStrdup ("<div></div>");
+
+	xmlFreeDoc (doc);
 	return result;
 }
 
@@ -296,6 +282,7 @@ xhtml_strip_unsupported_tags (const gchar *html)
 {
 	if (!unsupported_tag_strippers) {
 		xhtml_stripper_add(&unsupported_tag_strippers, "<\\s*/?wbr[^>]*/?\\s*>");
+		xhtml_stripper_add(&unsupported_tag_strippers, "<\\s*/?body[^>]*/?\\s*>");
 	}
 	
 	return xhtml_strip(html, unsupported_tag_strippers);
