@@ -1,7 +1,7 @@
 /**
  * @file xml.h  XML helper methods for Liferea
  * 
- * Copyright (C) 2003-2010  Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2003-2010  Lars Windolf <lars.lindner@gmail.com>
  * Copyright (C) 2004-2006  Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -55,6 +55,16 @@ gchar * unhtmlize (gchar *string);
  */
 gchar * unxmlize (gchar *string);
 
+/**
+ * Extract XHTML from a string of HTML and place it in a div tag.
+ *
+ * @param html		The HTML
+ * @param nodeBase	An URI to set as xml:base, or #NULL
+ *
+ * @returns XHTML version of the HTML
+ */
+gchar * xhtml_extract_from_string (const gchar *html, const gchar *nodeBase);
+
 /** 
  * Extract XHTML from the children of the passed node.
  *
@@ -86,17 +96,6 @@ gchar * xhtml_strip_dhtml (const gchar *html);
  */
 gchar * xhtml_strip_unsupported_tags (const gchar *html);
 
-/**
- * Convert the given string to proper XHTML content.
- * Note: this function does not respect relative URLs
- * and is to be used for cache migration 1.0 -> 1.1 only!
- *
- * @param text		usually an entity escaped HTML string
- *
- * @returns a new valid XHTML string
- */
-gchar * xhtml_from_text (const gchar *text);
-
 /**
  * Checks the given string for XHTML well formedness.
  *
@@ -114,7 +113,7 @@ gboolean xhtml_is_well_formed (const gchar *text);
  */
 xmlNodePtr xpath_find (xmlNodePtr node, const gchar *expr);
 
-/** Function type used by common_xpath_foreach_match() */
+/** Function type used by xpath_foreach_match() */
 typedef void (*xpathMatchFunc)(xmlNodePtr match, gpointer user_data);
 
 /**
